@@ -21,13 +21,18 @@
  *
  */
 
-#include "gmime-utils.h"
-#include "gmime-part.h"
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
+
+#include "gmime-utils.h"
+#include "gmime-table-private.h"
+#include "gmime-part.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#include <langinfo.h>
 
 #define d(x)
 
@@ -35,6 +40,10 @@
 
 #define GMIME_FOLD_LEN  76
 
+#ifndef HAVE_ISBLANK
+#define isblank(c) (c == ' ' || c == '\t')
+#endif
+
 static char *base64_alphabet =
 "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 
@@ -43,25 +52,6 @@ static unsigned char tohex[16] = {
 	'8', '9', 'A', 'B', 'C', 'D', 'E', 'F'
 };
 
-static unsigned short gmime_special_table[256] = {
-	  5,  5,  5,  5,  5,  5,  5,  5,  5,231,  7,  5,  5, 39,  5,  5,
-	  5,  5,  5,  5,  5,  5,  5,  5,  5,  5,  5,  5,  5,  5,  5,  5,
-	242,448, 76,192,192,192,192,192, 76, 76,448,448, 76,448, 72,324,
-	448,448,448,448,448,448,448,448,448,448, 76, 76, 76,  4, 76, 68,
-	 76,448,448,448,448,448,448,448,448,448,448,448,448,448,448,448,
-	448,448,448,448,448,448,448,448,448,448,448,108,236,108,192, 64,
-	192,448,448,448,448,448,448,448,448,448,448,448,448,448,448,448,
-	448,448,448,448,448,448,448,448,448,448,448,192,192,192,192,  5,
-	  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
-	  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
-	  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
-	  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
-	  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
-	  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
-	  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
-	  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
-};
-
 static unsigned char gmime_base64_rank[256] = {
 	255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
 	255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
@@ -101,109 +91,6 @@ static unsigned char gmime_uu_rank[256] = {
 };
 
 
-enum {
-	IS_CTRL		= 1<<0,
-	IS_LWSP		= 1<<1,
-	IS_TSPECIAL	= 1<<2,
-	IS_SPECIAL	= 1<<3,
-	IS_SPACE	= 1<<4,
-	IS_DSPECIAL	= 1<<5,
-	IS_QPSAFE	= 1<<6,
-	IS_ESAFE	= 1<<7,	/* encoded word safe */
-	IS_PSAFE	= 1<<8,	/* encoded word in phrase safe */
-};
-
-#define is_ctrl(x) ((gmime_special_table[(unsigned char)(x)] & IS_CTRL) != 0)
-#define is_lwsp(x) ((gmime_special_table[(unsigned char)(x)] & IS_LWSP) != 0)
-#define is_tspecial(x) ((gmime_special_table[(unsigned char)(x)] & IS_TSPECIAL) != 0)
-#define is_type(x, t) ((gmime_special_table[(unsigned char)(x)] & (t)) != 0)
-#define is_ttoken(x) ((gmime_special_table[(unsigned char)(x)] & (IS_TSPECIAL|IS_LWSP|IS_CTRL)) == 0)
-#define is_atom(x) ((gmime_special_table[(unsigned char)(x)] & (IS_SPECIAL|IS_SPACE|IS_CTRL)) == 0)
-#define is_dtext(x) ((gmime_special_table[(unsigned char)(x)] & IS_DSPECIAL) == 0)
-#define is_fieldname(x) ((gmime_special_table[(unsigned char)(x)] & (IS_CTRL|IS_SPACE)) == 0)
-#define is_qpsafe(x) ((gmime_special_table[(unsigned char)(x)] & IS_QPSAFE) != 0)
-#define is_especial(x) ((gmime_special_table[(unsigned char)(x)] & IS_ESPECIAL) != 0)
-#define is_psafe(x) ((gmime_special_table[(unsigned char)(x)] & IS_PSAFE) != 0)
-
-#ifndef HAVE_ISBLANK
-#define isblank(c) ((c) == ' ' || (c) == '\t')
-#endif /* HAVE_ISBLANK */
-
-#define CHARS_LWSP " \t\n\r"               /* linear whitespace chars */
-#define CHARS_TSPECIAL "()<>@,;:\\\"/[]?="
-#define CHARS_SPECIAL "()<>@,;:\\\".[]"
-#define CHARS_CSPECIAL "()\\\r"	           /* not in comments */
-#define CHARS_DSPECIAL "[]\\\r \t"	   /* not in domains */
-#define CHARS_ESPECIAL "()<>@,;:\"/[]?.="  /* encoded word specials (rfc2047 5.1) */
-#define CHARS_PSPECIAL "!*+-/"             /* encoded phrase specials (rfc2047 5.3) */
-
-#ifdef BUILD_TABLE
-/* code to rebuild the gmime_special_table */
-static void
-header_remove_bits (gushort bit, guchar *vals)
-{
-	gint i;
-	
-	for (i = 0; vals[i]; i++)
-		gmime_special_table[vals[i]] &= ~bit;
-}
-
-static void
-header_init_bits (gushort bit, gushort bitcopy, gboolean remove, guchar *vals)
-{
-	gint i, len = strlen (vals);
-	
-	if (!remove) {
-		for (i = 0; i < len; i++) {
-			gmime_special_table[vals[i]] |= bit;
-		}
-		if (bitcopy) {
-			for (i = 0; i < 256; i++) {
-				if (gmime_special_table[i] & bitcopy)
-					gmime_special_table[i] |= bit;
-			}
-		}
-	} else {
-		for (i = 0; i < 256; i++)
-			gmime_special_table[i] |= bit;
-		for (i = 0; i < len; i++) {
-			gmime_special_table[vals[i]] &= ~bit;
-		}
-		if (bitcopy) {
-			for (i = 0; i < 256; i++) {
-				if (gmime_special_table[i] & bitcopy)
-					gmime_special_table[i] &= ~bit;
-			}
-		}
-	}
-}
-
-static void
-header_decode_init (void)
-{
-	gint i;
-	
-	for (i = 0; i < 256; i++) {
-		gmime_special_table[i] = 0;
-		if (i < 32)
-			gmime_special_table[i] |= IS_CTRL;
-		if ((i >= 33 && i <= 60) || (i >= 62 && i <= 126) || i == 32 || i == 9)
-			gmime_special_table[i] |= (IS_QPSAFE | IS_ESAFE);
-		if ((i >= '0' && i <= '9') || (i >= 'a' && i <= 'z') || (i >= 'A' && i <= 'Z'))
-			gmime_special_table[i] |= IS_PSAFE;
-	}
-	
-	gmime_special_table[127] |= IS_CTRL;
-	gmime_special_table[' '] |= IS_SPACE;
-	header_init_bits (IS_LWSP, 0, FALSE, CHARS_LWSP);
-	header_init_bits (IS_TSPECIAL, IS_CTRL, FALSE, CHARS_TSPECIAL);
-	header_init_bits (IS_SPECIAL, 0, FALSE, CHARS_SPECIAL);
-	header_init_bits (IS_DSPECIAL, 0, FALSE, CHARS_DSPECIAL);
-	header_remove_bits (IS_ESAFE, CHARS_ESPECIAL);
-	header_init_bits (IS_PSAFE, 0, FALSE, CHARS_PSPECIAL);
-}
-#endif /* BUILD_TABLE */
-
 /* hrm, is there a library for this shit? */
 static struct {
 	char *name;
@@ -235,6 +122,20 @@ static char *tm_days[] = {
 	"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"
 };
 
+#define DEFAULT_CODESET "iso-8859-1"
+
+static const gchar*
+get_codeset (void)
+{
+	const gchar * codeset;
+
+	codeset = nl_langinfo (CODESET);
+	if (!codeset || !*codeset)
+		codeset = DEFAULT_CODESET;
+
+	return codeset;
+}
+
 
 /**
  * g_mime_utils_header_format_date:
@@ -406,8 +307,7 @@ get_time (const gchar *in, gint *hour, gint *min, gint *sec)
 			digits = FALSE;
 	}
 	
-/* Ameol software doesn't put the seconds on the time */
-
+	/* Ameol software doesn't put the seconds on the time */
 	if (!digits || (colons != 2 && colons != 1))
 		return FALSE;
 	
@@ -655,6 +555,28 @@ g_mime_utils_header_printf (const gchar *format, ...)
 	return ret;
 }
 
+static gboolean
+need_quotes (const char *string)
+{
+	gboolean quoted = FALSE;
+	const char *inptr;
+	
+	inptr = string;
+	
+	while (*inptr) {
+		if (*inptr == '\\')
+			inptr++;
+		else if (*inptr == '"')
+			quoted = !quoted;
+		else if (!quoted && (is_tspecial (*inptr) || *inptr == '.'))
+			return TRUE;
+		
+		if (*inptr)
+			inptr++;
+	}
+	
+	return FALSE;
+}
 
 /**
  * g_mime_utils_quote_string: Quote a string.
@@ -668,18 +590,16 @@ g_mime_utils_header_printf (const gchar *format, ...)
 gchar *
 g_mime_utils_quote_string (const gchar *string)
 {
-	GString *out;
+	gboolean quote;
+	const gchar *c;
 	gchar *qstring;
-	guchar *c;
-	gboolean quote = FALSE;
+	GString *out;
 	
 	out = g_string_new ("");
+	quote = need_quotes (string);
 	
-	for (c = (guchar *) string; *c; c++) {
-		if (is_tspecial (*c))
-			quote = TRUE;
-		
-		if (*c == '"' || *c == '\\')
+	for (c = string; *c; c++) {
+		if ((*c == '"' && quote) || *c == '\\')
 			g_string_append_c (out, '\\');
 		
 		g_string_append_c (out, *c);
@@ -832,23 +752,21 @@ quoted_decode (const guchar *in, gint len, guchar *out)
 	return -1;
 }
 
+static gboolean
+is_8bit_word_encoded (const guchar * word)
+{
+	guint len = strlen (word);
+	return len>=7 && word[0]=='=' && word[1]=='?' && word[len-2]=='?' && word[len-1]=='=';
+}
+
 static guchar *
-decode_8bit_word (const guchar *word, gboolean *was_encoded)
+decode_encoded_8bit_word (const guchar *word)
 {
 	guchar *inptr, *inend;
 	guint len;
 	
 	len = strlen (word);
 	
-	*was_encoded = TRUE;
-	
-	/* just make sure this is valid input */
-	if (len < 7 || !(word[0] == '=' && word[1] == '?' && word[len - 2] == '?' && word[len - 1] == '=')) {
-		d(fprintf (stderr, "invalid\n"));
-		*was_encoded = FALSE;
-		return g_strdup (word);
-	}
-	
 	inptr = (guchar *) word + 2;
 	inend = (guchar *) word + len - 2;
 	
@@ -908,9 +826,9 @@ g_mime_utils_8bit_header_decode (const guchar *in)
 	gboolean last_was_encoded = FALSE;
 	gboolean last_was_space = FALSE;
 	
-	out = g_string_new ("");
-	whtspc = g_string_new ("");
-	word = g_string_new ("");
+	out = g_string_sized_new (256);
+	whtspc = g_string_sized_new (256);
+	word = g_string_sized_new (256);
 	inptr = in;
 	
 	while (inptr && *inptr) {
@@ -919,10 +837,15 @@ g_mime_utils_8bit_header_decode (const guchar *in)
 		if (isspace (c) && !last_was_space) {
 			/* we reached the end of a 'word' */
 			gboolean was_encoded;
-			guchar *dword;
+			const gchar *cword;
+			guchar *dword = NULL;
+
+			if ((was_encoded = is_8bit_word_encoded (word->str)))
+				cword = dword = decode_encoded_8bit_word (word->str);
+			else
+				cword = word->str;
 			
-			dword = decode_8bit_word (word->str, &was_encoded);
-			if (dword) {
+			if (cword) {
 				if (!(last_was_encoded && was_encoded)) {
 					/* rfc2047 states that you
                                            must ignore all whitespace
@@ -930,7 +853,7 @@ g_mime_utils_8bit_header_decode (const guchar *in)
 					g_string_append (out, whtspc->str);
 				}
 				
-				g_string_append (out, dword);
+				g_string_append (out, cword);
 				g_free (dword);
 			} else {
 				was_encoded = FALSE;
@@ -939,12 +862,9 @@ g_mime_utils_8bit_header_decode (const guchar *in)
 			}
 			
 			last_was_encoded = was_encoded;
-			
-			g_string_free (whtspc, TRUE);
-			whtspc = g_string_new ("");
-			
-			g_string_free (word, TRUE);
-			word = g_string_new ("");
+
+			g_string_truncate (whtspc, 0);
+			g_string_truncate (word, 0);
 		}
 		
 		if (isspace (c)) {
@@ -958,10 +878,15 @@ g_mime_utils_8bit_header_decode (const guchar *in)
 	
 	if (word->len || whtspc->len) {
 		gboolean was_encoded;
-		guchar *dword;
+		const guchar *cword;
+		guchar *dword = NULL;
+
+		if ((was_encoded = is_8bit_word_encoded (word->str)))
+			cword = dword = decode_encoded_8bit_word (word->str);
+		else
+			cword = word->str;
 		
-		dword = decode_8bit_word (word->str, &was_encoded);
-		if (dword) {
+		if (cword) {
 			if (!(last_was_encoded && was_encoded)) {
 				/* rfc2047 states that you
 				   must ignore all whitespace
@@ -969,7 +894,7 @@ g_mime_utils_8bit_header_decode (const guchar *in)
 				g_string_append (out, whtspc->str);
 			}
 			
-			g_string_append (out, dword);
+			g_string_append (out, cword);
 			g_free (dword);
 		} else {
 			g_string_append (out, whtspc->str);
@@ -1000,9 +925,9 @@ quoted_encode (const guchar *in, gint len, guchar *out, gushort safemask)
 	
 	while (inptr < inend) {
 		c = *inptr++;
-		if (gmime_special_table[c] & safemask && c != '\t') {
-			if (c == ' ')
-				c = '_';
+		if (c == ' ') {
+			*outptr++ = '_';
+		} else if (gmime_special_table[c] & safemask) {
 			*outptr++ = c;
 		} else {
 			*outptr++ = '=';
@@ -1060,8 +985,8 @@ encode_8bit_word (const guchar *word, gushort safemask, gboolean *this_was_encod
 	
 	if (this_was_encoded)
 		*this_was_encoded = TRUE;
-	
-	return g_strdup_printf ("=?iso-8859-1?%c?%s?=", encoding, encoded);
+
+	return g_strdup_printf ("=?%s?%c?%s=", get_codeset(), encoding, encoded);	
 }
 
 
@@ -1122,10 +1047,8 @@ g_mime_utils_8bit_header_encode (const guchar *in)
 				ewhtspc = alloca (whtspc->len * 3 + 4);
 				len = quoted_encode (whtspc->str, whtspc->len, ewhtspc, IS_ESAFE);
 				ewhtspc[len] = '\0';
-				
-				g_string_append (out, " =?iso-8859-1?q?");
-				g_string_append (out, ewhtspc);
-				g_string_append (out, "?= ");
+		
+				g_string_sprintfa (out, " =?%s?q?%s?= ", get_codeset(), ewhtspc);
 			} else {
 				g_string_append (out, whtspc->str);
 			}
@@ -1133,12 +1056,9 @@ g_mime_utils_8bit_header_encode (const guchar *in)
 			/* append the encoded word */
 			g_string_append (out, eword);
 			g_free (eword);
-			
-			g_string_free (whtspc, TRUE);
-			whtspc = g_string_new ("");
-			
-			g_string_free (word, TRUE);
-			word = g_string_new ("");
+
+			g_string_truncate (whtspc, 0);
+			g_string_truncate (word, 0);
 			
 			last_was_encoded = this_was_encoded;
 			is8bit = FALSE;
@@ -1174,10 +1094,8 @@ g_mime_utils_8bit_header_encode (const guchar *in)
 			ewhtspc = alloca (whtspc->len * 3 + 4);
 			len = quoted_encode (whtspc->str, whtspc->len, ewhtspc, IS_ESAFE);
 			ewhtspc[len] = '\0';
-			
-			g_string_append (out, " =?iso-8859-1?q?");
-			g_string_append (out, ewhtspc);
-			g_string_append (out, "?= ");
+		
+			g_string_sprintfa (out, " =?%s?q?%s?= ", get_codeset(), ewhtspc);
 		} else {
 			g_string_append (out, whtspc->str);
 		}
