@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000  Pan Development Team <pan@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -54,10 +54,18 @@ dialog_about (GtkWidget * window)
 	                   "http://pan.rebelbase.com/");
 
 	a = PAN_ABOUT(w);
-	pan_about_add_author (a, "Charles Kerr", _("Lead Programmer"), "charles@rebelbase.com");
-	pan_about_add_author (a, "Matt Eagleson", _("Programmer & Project Originator"), "e.Messiah@superpimp.org");
-	pan_about_add_author (a, "Tov are Jacobsen", _("Translations & PR"), "tovj@stud.ntnu.no");
-	pan_about_add_author (a, "Jason Leach", _("Programmer, 0.6 - 0.7"), "elerium@superpimp.org");
+	pan_about_add_author (a, "Charles Kerr", "charles@rebelbase.com",
+	                      _("Lead Programmer"));
+	pan_about_add_author (a, "Christophe Lambin", "clambin@easynet.be",
+	                      _("Programmer"));
+	pan_about_add_author (a, "Adam Huffman", "bloch@verdurin.com",
+	                      _("Programmer & Documentation"));
+	pan_about_add_author (a, "Tov are Jacobsen", "tovj@stud.ntnu.no",
+	                      _("Translation Coordinator"));
+	pan_about_add_author (a, "Matt Eagleson", "e.Messiah@superpimp.org",
+	                      _("Project Originator, Programmer"));
+	pan_about_add_author (a, "Jason Leach", "jleach@ximian.com",
+	                      _("Programmer, 0.6 - 0.7"));
 
 	credits_i18n = get_credits_translators ();
 	credits_code = get_credits_code ();
@@ -101,18 +109,17 @@ dialog_about (GtkWidget * window)
 static gchar *
 get_credits_translators (void)
 {
-	return g_strdup ("\n\tDanish - Keld Simonsen\n\tFinnish - Jarmo Kielosto\n\tFrench - Christophe Merlet\n\tGerman - Christian Meyer\n\tItalian - Francesco Lamonica\n\tJapanese - Takeshi Aihana\n\tKorean - Nam SungHyun\n\tTurkish - Grkem etin");
+	return g_strdup ("\n\tAntonio de la Torre \n\tJarmo Kielosto \n\tAndrs Tmr \n\tMarcel Telka \n\tChristian Rose");
 }
 
 static gchar *
 get_credits_code (void)
 {
-	return g_strdup ("\n\tChristophe Lambin\n\tSergey Pinaev");
+	return g_strdup ("\n\tVolodymyr M . Lisivka\n\tFredrik Pettersson\n\tMike Robinson\n\tOwen Taylor");
 }
 
 static gchar *
 get_credits_users (void)
 {
-	return g_strdup ("\n\tFabrizio Ammollo\n\tSndor Brny\n\tJasper Berlijn\n\tMystic Zen Biker Twigboy\n\tJacob Ils Christensen\n\tCyber Czar\n\tWendell Dingus\n\tMagdalena Fabrykowska\n\tGauze\n\tKevin L. Gottsman\n\tAdam Huffman\n\tGarry Knight\n\tJohn J. LeMay Jr.\n\tChristian Lohmaier\n\tMorten Breum Mikkelsen\n\tSven Neuhaus\n\tEric Ortega\n\tMarvin R. Pierce\n\tWilliam Peterson\n\tSean\n\tDaniel Shackelford\n\tDietmar Staab\n\tFelix Tilley\n\tAdam Warner");
+	return g_strdup ("\n\tAnubis \n\tJed S. Baer \n\tGavin Baker \n\tSteven Behnke \n\tAdrian Von Bidder \n\tCN \n\tDuncan Coutts \n\tRyan Daniels \n\tTrey Egan \n\tPaolo Fiore \n\tJim Henderson \n\tDave Hill \n\tPhilip Van Hoof \n\tSimon Jester \n\tFrancesco Lamonica \n\tLenny \n\tShawn Lindsay \n\tToralf Lund \n\tMatt \n\tmdew \n\tJohn Mellor \n\tDavid Oberbeck \n\tHuub Reuver \n\tNoah Romer \n\tB J Rosen \n\tAlan Schmidt \n\tFlorian Schmidt \n\tT'aZ \n\tClark Tompsett \n\tWilliam Shotts \n\tDavid Stuart \n\tSam Simpson \n\tC Sullivan \n\tJohn Summerfield \n\tTester \n\tMystic Zen Biker Twigboy \n\tWanderer \n\tAndy Zbikowski \n\tTaylor James Zingsheim");
 }
-
