@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 2000  Pan Development Team <pan@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,13 +29,11 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <glib.h>
-#include <gtk/gtk.h>
-#include <libgnome/libgnome.h>
-#include <libgnomeui/libgnomeui.h>
+#include <gnome.h>
+
+#include <pan/base/group.h>
 
 #include <pan/globals.h>
-#include <pan/group.h>
 #include <pan/queue.h>
 #include <pan/task-headers.h>
 #include <pan/util.h>
@@ -164,13 +162,12 @@ dialog_download_headers (Group * group)
 	dialog = g_new0 (DownloadHeadersDialogStruct, 1);
 	dialog->group = group;
 	pan_lock ();
-	s = g_strdup_printf (
-		_("Download Headers for group `%s'"),
-		group_get_readable_name(group));
+	s = g_strdup_printf (_("Pan: Download Headers for group `%s'"),
+	                     group_get_readable_name(group));
 	dialog->dialog = gnome_dialog_new (s,
-		GNOME_STOCK_BUTTON_OK,
-		GNOME_STOCK_BUTTON_CANCEL,
-		NULL);
+	                                   GNOME_STOCK_BUTTON_OK,
+	                                   GNOME_STOCK_BUTTON_CANCEL,
+	                                   NULL);
 	g_free (s);
 
 	/**
