@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000  Pan Development Team <pan@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,22 +20,19 @@
 
 #include <config.h>
 
-#include <glib.h>
-#include <gtk/gtk.h>
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
-#include <libgnomeui/gnome-app.h>
-#include <libgnomeui/gnome-app-helper.h>
-#include <libgnomeui/gnome-stock.h>
+#include <gnome.h>
 
-#include "pan/globals.h"
-#include "pan/debug.h"
-#include "pan/gui.h"
-#include "pan/log.h"
-#include "pan/prefs.h"
-#include "pan/util.h"
-#include "pan/dialogs/dialogs.h"
-#include "pan/xpm/caution.xpm"
+#include <pan/base/debug.h>
+#include <pan/base/log.h>
+#include <pan/base/pan-glib-extensions.h>
+#include <pan/base/util-file.h>
+
+#include <pan/globals.h>
+#include <pan/gui.h>
+#include <pan/prefs.h>
+#include <pan/util.h>
+#include <pan/dialogs/dialogs.h>
+#include <pan/xpm/caution.xpm>
 
 typedef struct
 {
