@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000  Pan Development Team <pan@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,8 +32,11 @@
 #include <gnome.h>
 #include <gdk_imlib.h>
 
+#include <pan/base/serverlist.h>
+
 #include <pan/globals.h>
 #include <pan/pan.h>
+#include <pan/gui.h>
 #include <pan/prefs.h>
 #include <pan/util.h>
 
@@ -100,6 +103,7 @@ druid_finish (GnomeDruidPage  * page,
               GnomeDruid      * druid,
               gpointer          window)
 {
+	Server * server = NULL;
 	GString * buf = g_string_new (NULL);
 	Druid * ddata
 		= gtk_object_get_data (GTK_OBJECT (window), "data");
@@ -126,11 +130,9 @@ druid_finish (GnomeDruidPage  * page,
 	const gchar * smtp_port
 		= gtk_entry_get_text (GTK_ENTRY(ddata->smtp_port));
 						
-
-	
 	if (profile_name && server_address)
 	{
-		Server * server = server_new ();
+		server = server_new ();
                 server->name = g_strdup (profile_name);
 		server->address = g_strdup (server_address);
                 server->port = server_port;
@@ -140,7 +142,7 @@ druid_finish (GnomeDruidPage  * page,
                 server->gen_msgid = TRUE;
 		server->max_connections = 4;
 		server->idle_secs_before_timeout = 180;
-		server_add_new_server (server);
+		serverlist_add_server (server);
 		gnome_config_set_string ("/Pan/State/Server", profile_name);
 	}
 
@@ -154,8 +156,9 @@ druid_finish (GnomeDruidPage  * page,
 	
 	/* start Pan proper */
 	gnome_config_sync ();
-	pan_init ();
+	gui_construct (NULL);
 	gtk_widget_show_all (Pan.window);
+	serverlist_set_active_server (server);
 
 	/* cleanup */
 	g_free (ddata);
@@ -567,7 +570,7 @@ dialog_newuser (void)
 	page_e = gnome_druid_page_standard_new_with_vals (_("Save Profile"), logo);
 	page_finish = gnome_druid_page_finish_new_with_vals (
 		_("Congratulations"),
-		_("You have successfullly entered all of\nthe information required to set up Pan.\n\nTo save these settings, click Finish."),
+		_("You have successfully entered all of\nthe information required to set up Pan.\n\nTo save these settings, click Finish."),
 		logo,
 		watermark);
 
