@@ -1,5 +1,5 @@
 /* Pan "About" Dialog
- * Copyright (C) 2001 Charles Kerr <charles@superpimp.org>
+ * Copyright (C) 2001 Charles Kerr <charles@rebelbase.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -316,8 +316,8 @@ pan_add_someone (GtkText          ** text,
 void
 pan_about_add_author   (PanAbout      * about,
                         const gchar   * name,
-                        const gchar   * job_title,
-                        const gchar   * email)
+                        const gchar   * email,
+                        const gchar   * job_title)
 {
 	Impl * impl;
 
@@ -334,8 +334,8 @@ pan_about_add_author   (PanAbout      * about,
 void
 pan_about_add_credit   (PanAbout      * about,
                         const gchar   * name,
-                        const gchar   * job_title,
-                        const gchar   * email)
+                        const gchar   * email,
+                        const gchar   * job_title)
 {
 	Impl * impl;
 
