@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team <pan@rebelbase.com>
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,29 +21,29 @@
 #include <config.h>
 #include <errno.h>
 
-#include <glib.h>
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
-#include <libgnome/gnome-config.h>
-#include <libgnomeui/libgnomeui.h>
+#include <gnome.h>
 
-#include "articlelist.h"
-#include "debug.h"
-#include "dialogs/dialogs.h"
-#include "file-grouplist.h"
-#include "fnmatch.h"
-#include "globals.h"
-#include "grouplist.h"
-#include "gui.h"
-#include "prefs.h"
-#include "queue.h"
-#include "task-group-count.h"
-#include "task-headers.h"
-#include "task-grouplist.h"
-#include "util.h"
+#include <pan/base/debug.h>
+#include <pan/base/file-grouplist.h>
+#include <pan/base/fnmatch.h>
+#include <pan/base/pan-glib-extensions.h>
+#include <pan/base/serverlist.h>
+
+#include <pan/articlelist.h>
+#include <pan/dialogs/dialogs.h>
+#include <pan/globals.h>
+#include <pan/grouplist.h>
+#include <pan/group-ui.h>
+#include <pan/gui.h>
+#include <pan/prefs.h>
+#include <pan/queue.h>
+#include <pan/task-group-count.h>
+#include <pan/task-bodies.h>
+#include <pan/task-headers.h>
+#include <pan/task-grouplist.h>
+#include <pan/util.h>
 
 #include "xpm/folder.xpm"
-#include "xpm/envelope.xpm"
 
 static void grouplist_update_ui_thread (void* data);
 static void group_menu_popup (GdkEventButton * event);
@@ -52,7 +52,7 @@ static void grouplist_refresh_filter (void);
 
 extern GtkWidget   * ttips;
 static GnomePixmap * folder_pixmap           = NULL;
-static GnomePixmap * group_pixmap            = NULL;
+static GnomePixmap * subscribed_pixmap       = NULL;
 
 static int           group_mode              = GROUP_ALL;
 static GtkWidget   * group_clist_menu        = NULL;
@@ -91,7 +91,7 @@ static Server *
 grouplist_get_visible_server (void)
 {
 	return group_mode==GROUP_FOLDERS
-		? server_get_by_name(INTERNAL_SERVER_NAME)
+		? serverlist_get_named_server(INTERNAL_SERVER_NAME)
 		: my_server;
 }
 
@@ -105,17 +105,16 @@ static void
 group_dialog_clicked_cb (GnomeDialog * dialog, int num, Server * server)
 {
 	if (num==0) /* yes, get list */
-		queue_add (TASK(task_grouplist_new (server,
-		                                    TRUE,
-		                                    GROUPLIST_ALL)));
+		queue_add (TASK(task_grouplist_new (server, GROUPLIST_ALL)));
 }
 
-void
+static void
 grouplist_set_server (Server * server)
 {
 	Server * old_server = my_server;
 	debug_enter ("grouplist_set_server");
 
+
 	g_return_if_fail (server != NULL);
 	if (my_server == server)
 		return;
@@ -130,18 +129,11 @@ grouplist_set_server (Server * server)
 		pan_unlock ();
 	}
 
-	if (my_server->groups->len) {
+	if (file_grouplist_exists (my_server))
+	{
 		grouplist_update_ui (my_server);
-		return;
 	}
-
-	/* haven't loaded the groups, but found a file for it */
-	if (file_grouplist_exists (my_server)) {
-		grouplist_load (my_server);
-		return;
-	}
-
-	/* can't find the group list for this server */
+	else
 	{
 		GtkWidget * w;
 		gchar * s = g_strdup_printf (
@@ -312,30 +304,6 @@ select_row_cb (GtkWidget  * widget,
 ******
 *****/
 
-GPtrArray*
-grouplist_get_subscribed_groups (void)
-{
-	GPtrArray * a;
-	debug_enter ("grouplist_get_subscribed_groups");
-       
-	a = g_ptr_array_new ();
-
-	if (my_server != NULL)
-	{
-		guint i;
-		pan_g_ptr_array_reserve (a, my_server->groups->len);
-		for (i=0; i!=my_server->groups->len; ++i)
-		{
-			Group * g = GROUP(g_ptr_array_index(my_server->groups,i));
-			if (group_is_subscribed (g))
-				g_ptr_array_add (a, g);
-		}
-	}
-
-	debug_exit ("grouplist_get_subscribed_groups");
-	return a;
-}
-
 void
 grouplist_set_selected_groups (const GPtrArray * a)
 {
@@ -403,12 +371,13 @@ grouplist_get_selected_groups (void)
 	pan_lock ();
 	for (l=clist->selection; l!=NULL; l=l->next)
 		g_ptr_array_add (retval, gtk_clist_get_row_data (clist, GPOINTER_TO_INT (l->data)));
+	pan_unlock ();
+
 	if (!retval->len) {
 		Group * group = articlelist_get_group ();
 		if (group != NULL)
 			g_ptr_array_add (retval, group);
 	}
-	pan_unlock ();
 
 	debug_exit ("grouplist_get_selected_groups");
 	return retval;
@@ -418,11 +387,33 @@ grouplist_get_selected_groups (void)
 ******
 *****/
 
+static void
+group_download_all_articles (Group * group)
+{
+	Task * task = NULL;
+	GPtrArray * a = NULL;
+
+	g_return_if_fail (group != NULL);
+	g_return_if_fail (group->_articles != NULL);
+
+	group_ref_articles (group, NULL);
+	a = group_get_article_array (group);
+	if (a->len)
+		task = TASK(task_bodies_new (group, a));
+	if (task != NULL)
+		queue_add (task);
+	group_unref_articles (group, NULL);
+
+	/* cleanup */
+	g_ptr_array_free (a, TRUE);
+}
+
 void
 grouplist_remove_row (const Group *group)
 {
 	debug_enter ("grouplist_remove_row");
-	if (group->server == my_server)
+
+	if (group->server == grouplist_get_visible_server())
 	{
 		GtkCList *list = GTK_CLIST(Pan.group_clist);
 		int row;
@@ -435,6 +426,7 @@ grouplist_remove_row (const Group *group)
 		pan_unlock();
 		g_static_mutex_unlock (&grouplist_ui_mutex);
 	}
+
 	debug_exit ("grouplist_remove_row");
 }
 static void
@@ -576,15 +568,13 @@ grouplist_selected_mark_read (void)
 *****/
 
 static void
-delete_groups_dialog_clicked_cb (GnomeDialog      * dialog,
-                                 int                button,
-                                 gpointer           user_data)
+delete_groups_dialog_cb (gint reply, gpointer data)
 {
-	GPtrArray * a = (GPtrArray*) user_data;
-	debug_enter ("delete_groups_dialog_clicked_cb");
+	GPtrArray * a;
+	debug_enter ("delete_groups_dialog_cb");
 
-	/* if user clicked yes; destroy the groups */
-	if (button == 0)
+	a = (GPtrArray*) data;
+	if (reply == 0 /*yes*/)
 	{
 		guint i;
 		GtkCList * clist = GTK_CLIST(Pan.group_clist);
@@ -604,76 +594,26 @@ delete_groups_dialog_clicked_cb (GnomeDialog      * dialog,
 		pan_unlock ();
 	}
 
-	/* destroy the widget */
-	pan_lock();
-	gtk_widget_destroy (GTK_WIDGET(dialog));
-	pan_unlock();
-
-	debug_exit ("delete_groups_dialog_clicked_cb");
+	g_ptr_array_free (a, TRUE);
+	debug_exit ("delete_groups_dialog_cb");
 }
 
-
-
 void
 grouplist_selected_destroy (void)
 {
-	GPtrArray * a;
-	debug_enter ("grouplist_selected_destroy");
-
-       	a = grouplist_get_selected_groups ();
+	GPtrArray * a = grouplist_get_selected_groups ();
 	if (a->len != 0)
 	{
-		guint i;
-		gchar * warning;
-		GtkWidget * dialog;
-		GtkWidget * clist;
-		GtkWidget * scroll;
-		GtkWidget * label;
-
-		/* build the warning string */
-		warning = g_strdup(
-			_("This operation will delete the folders/groups and their articles!\n"
-			  " \n"
-			  "Do you REALLY want to do this?"));
-
-		/* make a warning dialog listing all the groups to be destroyed */
+		gchar * txt = g_strdup_printf (_("Are you sure you want to delete these %d groups/folders and their articles?"), (gint)a->len);
 		pan_lock ();
-		dialog = gnome_dialog_new (_("Delete Groups/Folders"),
-			GNOME_STOCK_BUTTON_YES,
-			GNOME_STOCK_BUTTON_NO,
-			NULL);
-		gtk_window_set_policy (GTK_WINDOW(dialog), TRUE, TRUE, FALSE);
-		label = gtk_label_new (warning);
-		g_free (warning);
-		gtk_misc_set_alignment (GTK_MISC(label), 0.0, 0.5);
-		gtk_label_set_justify (GTK_LABEL(label), GTK_JUSTIFY_LEFT);
-		gtk_box_pack_start (GTK_BOX(GNOME_DIALOG(dialog)->vbox),
-		                    label,
-		                    FALSE, TRUE, GNOME_PAD_SMALL);
-		gtk_signal_connect (GTK_OBJECT(dialog), "clicked",
-		                    GTK_SIGNAL_FUNC(delete_groups_dialog_clicked_cb), a);
-		clist = gtk_clist_new (1);
-		for (i=0; i!=a->len; ++i) {
-			Group * g = GROUP(g_ptr_array_index(a,i));
-			const gchar * name = group_get_readable_name (g);
-			gtk_clist_insert (GTK_CLIST(clist), -1, (gchar**)&name);
-		}
-		scroll = gtk_scrolled_window_new(NULL, NULL);
-	        gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(scroll),
-		                                GTK_POLICY_AUTOMATIC,
-		                                GTK_POLICY_AUTOMATIC);
-		gtk_container_add (GTK_CONTAINER(scroll), clist);
-		gtk_box_pack_start (GTK_BOX(GNOME_DIALOG(dialog)->vbox),
-		                    scroll,
-		                    TRUE, TRUE, GNOME_PAD_SMALL);
-		gtk_widget_set_usize (dialog, -2, 500);
-		gnome_dialog_set_parent (GNOME_DIALOG(dialog),
-		                         GTK_WINDOW(Pan.window));
-		gtk_widget_show_all (dialog);
+		gnome_question_dialog_parented (txt,
+		                                delete_groups_dialog_cb,
+		                                a,
+		                                GTK_WINDOW(Pan.window));
 		pan_unlock ();
+		g_free (txt);
 	}
-
-	debug_exit ("grouplist_selected_destroy");
+	else g_ptr_array_free (a, TRUE);
 }
 
 /*****
@@ -810,7 +750,7 @@ grouplist_subscribed_download_new (void)
 	GPtrArray * a;
 	debug_enter ("grouplist_subscribed_download_new");
 
-       	a = grouplist_get_subscribed_groups ();
+	a = server_get_groups (my_server, SERVER_GROUPS_SUBSCRIBED);
 	grouplist_start_task_headers (a, HEADERS_NEW, FALSE);
 	g_ptr_array_free (a, TRUE);
 
@@ -822,7 +762,7 @@ grouplist_subscribed_download_new_bodies (void)
 	GPtrArray * a;
 	debug_enter ("grouplist_subscribed_download_new_bodies");
 
-       	a = grouplist_get_subscribed_groups ();
+	a = server_get_groups (my_server, SERVER_GROUPS_SUBSCRIBED);
 	grouplist_start_task_headers (a, HEADERS_NEW, TRUE);
 	g_ptr_array_free (a, TRUE);
 
@@ -840,9 +780,7 @@ grouplist_get_all (void)
 	if (my_server == NULL)
 		pan_error_dialog ("No server selected.");
 	else
-		queue_add (TASK(task_grouplist_new (my_server,
-		                                    TRUE,
-		                                    GROUPLIST_ALL)));
+		queue_add (TASK(task_grouplist_new (my_server, GROUPLIST_ALL)));
 }
 
 void
@@ -851,9 +789,7 @@ grouplist_get_new (void)
 	if (my_server == NULL)
 		pan_error_dialog ("No server selected.");
 	else
-		queue_add (TASK(task_grouplist_new (my_server,
-		                                    TRUE,
-		                                    GROUPLIST_NEW)));
+		queue_add (TASK(task_grouplist_new (my_server, GROUPLIST_NEW)));
 }
 
 /*****
@@ -937,10 +873,11 @@ grouplist_mode_menu_create (void)
 
 	for (i=0; i<row_qty; ++i) {
 		GtkWidget *item = gtk_menu_item_new_with_label (foo[i].name);
+		gtk_widget_show (item);
+		gtk_menu_append (GTK_MENU (menu), item);
 		gtk_signal_connect (GTK_OBJECT(item), "activate",
 				    (GtkSignalFunc)grouplist_set_view_mode,
 				    GINT_TO_POINTER(foo[i].mode));
-		gtk_menu_append (GTK_MENU (menu), item);
 		if (group_mode == foo[i].mode)
 			index = i;
 	}
@@ -952,6 +889,60 @@ grouplist_mode_menu_create (void)
 	return option_menu;
 }
 
+/*****
+******  CALLBACKS
+*****/
+
+static gint
+grouplist_server_groups_added_cb (gpointer server, gpointer groups, gpointer foo)
+{
+	Server * visible;
+
+	debug_enter ("grouplist_server_groups_added_cb");
+
+	visible = grouplist_get_visible_server ();
+	if (visible!=NULL && visible==SERVER(server))
+		grouplist_update_ui (visible);
+
+	debug_exit ("grouplist_server_groups_added_cb");
+	return 0;
+}
+
+static gint
+grouplist_server_groups_removed_cb (gpointer server,
+                                    gpointer groups,
+                                    gpointer foo)
+{
+	Server * visible;
+
+	debug_enter ("grouplist_server_groups_removed_cb");
+
+	visible = grouplist_get_visible_server ();
+	if (visible!=NULL && visible==SERVER(server)) {
+		GPtrArray * a = (GPtrArray*) groups;
+		pan_g_ptr_array_foreach (a, (GFunc)grouplist_remove_row, NULL);
+	}
+
+	debug_exit ("grouplist_server_groups_removed_cb");
+	return 0;
+}
+
+static gint
+grouplist_group_changed_cb (gpointer group_data, gpointer unused1, gpointer unused2)
+{
+	const Group * group = GROUP(group_data);
+	if (group->server == grouplist_get_visible_server())
+		grouplist_update_groups (&group, 1);
+	return 0;
+}
+
+static gint
+grouplist_server_activated_cb (gpointer server_data, gpointer unused1, gpointer unused2)
+{
+	grouplist_set_server (SERVER(server_data));
+	return 0;
+}
+
 /*****
 ******
 *****/
@@ -959,7 +950,26 @@ grouplist_mode_menu_create (void)
 void
 grouplist_shutdown_module (void)
 {
+	/* stop listening... */
+	pan_callback_remove (serverlist_get_server_activated_callback(),
+	                     grouplist_server_activated_cb, NULL);
+	pan_callback_remove (group_get_group_changed_callback(),
+	                     grouplist_group_changed_cb, NULL);
+	pan_callback_remove (grouplist_group_selection_changed,
+	                     grouplist_selection_changed_cb, NULL);
+	pan_callback_remove (server_get_groups_added_callback(),
+	                     grouplist_server_groups_added_cb, NULL);
+	pan_callback_remove (server_get_groups_removed_callback(),
+	                     grouplist_server_groups_removed_cb, NULL);
+
+	/* save state for next time */
 	gnome_config_set_int ("/Pan/State/group_mode", group_mode);
+
+	/* remember server so that we can default to it next time */
+	if (my_server!=NULL && is_nonempty_string(my_server->name))
+		gnome_config_set_string ("/Pan/State/Server", my_server->name);
+
+	gnome_config_sync ();
 }
 
 gpointer
@@ -970,21 +980,32 @@ grouplist_create (void)
         GtkWidget * hbox;
 	GtkWidget * toolbar;
 	GtkCList * clist;
-
 	char * titles[5];
+	debug_enter ("grouplist_create");
+
 	titles[0] = "";
 	titles[1] = _("Groups");
 	titles[2] = _("Unread");
 	titles[3] = _("Total");
 	titles[4] = _("Description");
 
-       	toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL,GTK_TOOLBAR_TEXT);
-	grouplist_server_set_callback = pan_callback_new ();
+	/* create callbacks */
 	grouplist_group_selection_changed = pan_callback_new ();
+	grouplist_server_set_callback = pan_callback_new ();
 
+	/* register callbacks */
+	pan_callback_add (serverlist_get_server_activated_callback(),
+	                  grouplist_server_activated_cb, NULL);
+	pan_callback_add (group_get_group_changed_callback(),
+	                  grouplist_group_changed_cb, NULL);
 	pan_callback_add (grouplist_group_selection_changed,
-	                  grouplist_selection_changed_cb,
-	                  NULL);
+	                  grouplist_selection_changed_cb, NULL);
+	pan_callback_add (server_get_groups_added_callback(),
+	                  grouplist_server_groups_added_cb, NULL);
+	pan_callback_add (server_get_groups_removed_callback(),
+	                  grouplist_server_groups_removed_cb, NULL);
+
+       	toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL,GTK_TOOLBAR_TEXT);
 
 	vbox = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
 	gtk_container_set_border_width (GTK_CONTAINER(vbox), GNOME_PAD_SMALL); 
@@ -1000,7 +1021,7 @@ grouplist_create (void)
 	Pan.group_clist = gtk_clist_new_with_titles (5, titles);
 	clist = GTK_CLIST(Pan.group_clist);
 	clist->button_actions[1] = clist->button_actions[0];
-	widget_set_font (GTK_WIDGET(Pan.group_clist), grouplist_font);
+	pan_widget_set_font (GTK_WIDGET(Pan.group_clist), grouplist_font);
 	gtk_clist_set_column_justification (clist, 2, GTK_JUSTIFY_RIGHT);
 	gtk_clist_set_column_justification (clist, 3, GTK_JUSTIFY_RIGHT);
 
@@ -1010,7 +1031,6 @@ grouplist_create (void)
 	gtk_toolbar_append_space (GTK_TOOLBAR(toolbar));
 
 	grouplist_mode_menu = grouplist_mode_menu_create();
-	gtk_widget_set (grouplist_mode_menu, "width", 120, "height", 25, NULL); /* FIXME: If we don't do this the om doesn't show up*/
 	gtk_box_pack_start (GTK_BOX(hbox), grouplist_mode_menu, FALSE, FALSE, 0);
 
 	w = gtk_label_new (_("Find:"));
@@ -1077,9 +1097,16 @@ grouplist_create (void)
 
 	/* Lets have the Pixmap ready too */
 	folder_pixmap = (GnomePixmap*) gnome_pixmap_new_from_xpm_d (folder_xpm);
-	group_pixmap = (GnomePixmap*) gnome_pixmap_new_from_xpm_d (envelope_xpm);
+	subscribed_pixmap = GNOME_PIXMAP(gnome_stock_pixmap_widget_new (Pan.window, GNOME_STOCK_MENU_BOOK_RED));
+
+	/* bootstrap */
+	if (1) {
+		my_server = serverlist_get_active_server ();
+		grouplist_update_ui_thread (NULL);
+	}
 
 	gtk_widget_show_all (vbox);
+	debug_exit ("grouplist_create");
 	return vbox;
 }
 
@@ -1093,57 +1120,6 @@ grouplist_filter_changed_cb (void)
 ******
 *****/
 
-static gchar*
-grouplist_load_describe (const StatusItem* item)
-{
-	return g_strdup (_("Loading Groups"));
-}
-
-void
-grouplist_load (Server * server)
-{
-	StatusItem * status = NULL;
-	debug_enter ("grouplist_load");
-
-	/* entry assertions */
-	g_return_if_fail (server!=NULL);
-
-	/* create a new status item to get load messages */
-	status = STATUS_ITEM(status_item_new(grouplist_load_describe));
-	status_item_set_active (status, TRUE);
-
-	/* load the groups... */
-	file_grouplist_load (server, status);
-
-	/* remove the "load groups" status: grouplist_update_ui has its own */
-	status_item_set_active (status, FALSE);
-	pan_object_unref(PAN_OBJECT(status));
-	status = NULL;
-
-	debug_exit ("grouplist_load");
-}
-
-/*****
-******
-*****/
-
-static gboolean
-should_be_in_grouplist (const Server * server, const Group * group)
-{
-	g_return_val_if_fail (group!=NULL, FALSE);
-	g_return_val_if_fail (group->server!=NULL, FALSE);
-
-	if (group->server != server)
-		return FALSE;
-
-	if (group_is_folder(group) && group_mode!=GROUP_FOLDERS)
-		return FALSE;
-
-	if (group_mode!=GROUP_ALL && !(group_mode & group->flags))
-		return FALSE;
-
-	return TRUE;
-}
 static void
 grouplist_add_group_nolock (const Group * group)
 {
@@ -1178,7 +1154,7 @@ grouplist_add_group_nolock (const Group * group)
 	if (group_is_folder (group))
 		pixmap = folder_pixmap;
 	else if (group_is_subscribed (group))
-		pixmap = group_pixmap;
+		pixmap = subscribed_pixmap;
 	if (pixmap != NULL)
 		gtk_clist_set_pixmap (GTK_CLIST(Pan.group_clist), row, 0,
 		                      pixmap->pixmap,
@@ -1187,32 +1163,80 @@ grouplist_add_group_nolock (const Group * group)
 	/* cleanup */
 	g_free (name);
 }
+
 static gchar*
 grouplist_update_describe (const StatusItem *item)
 {
 	return g_strdup (_("Updating Group List"));
 }
+
 static void
-grouplist_update_ui_thread (void* data)
+grouplist_update_ui_thread (void * unused)
 {
 	gchar * title;
-	Server * server;
 	GtkCList * clist = GTK_CLIST(Pan.group_clist);
+	GPtrArray * groups;
 	GPtrArray * sel;
 	StatusItem *item = NULL;
 	int i;
 	debug_enter ("grouplist_update_ui_thread");
 
-       	server = grouplist_get_visible_server ();
-	g_return_if_fail (server != NULL);
-
-       	sel = grouplist_get_selected_groups ();
+	/**
+	***  Get the list of groups that we're going to add.
+	***
+	***  Note that we temporarily turn off the groups_added
+	***  callback -- the groups may be loaded as a result of
+	***  our call, and we don't want to update the clist twice.
+	***  This is a temporary workaround that should dissapear
+	***  in 0.10.x
+	**/
+	pan_callback_remove (server_get_groups_added_callback(),
+	                     grouplist_server_groups_added_cb, NULL);
+	if (group_mode == GROUP_SUBSCRIBED)
+	{
+		groups = my_server==NULL
+			? g_ptr_array_new ()
+			: server_get_groups (my_server, SERVER_GROUPS_SUBSCRIBED);
+	}
+	else if (group_mode == GROUP_ALL)
+	{
+		groups = my_server==NULL
+			? g_ptr_array_new ()
+			: server_get_groups (my_server, SERVER_GROUPS_ALL);
+	}
+	else if (group_mode == GROUP_FOLDERS)
+	{
+		Server * s = serverlist_get_named_server(INTERNAL_SERVER_NAME);
+		groups = s==NULL
+			? g_ptr_array_new()
+			: server_get_groups (s, SERVER_GROUPS_ALL);
+	}
+	else if (group_mode == GROUP_NEW)
+	{
+		groups = g_ptr_array_new ();
+		if (my_server != NULL) {
+			guint i;
+			GPtrArray * a = server_get_groups (my_server, SERVER_GROUPS_ALL);
+			for (i=0; i<a->len; ++i) {
+				Group * g = GROUP(g_ptr_array_index(a,i));
+				if (group_is_new (g))
+					g_ptr_array_add (groups, g);
+			}
+			g_ptr_array_free (a, TRUE);
+		}
+	}
+	else
+	{
+		pan_warn_if_reached ();
+		groups = g_ptr_array_new ();
+	}
+	pan_callback_add (server_get_groups_added_callback(),
+	                  grouplist_server_groups_added_cb, NULL);
 
         /* create a new status item to get sort/thread messages */
 	item = STATUS_ITEM(status_item_new(grouplist_update_describe));
 	status_item_set_active (item, TRUE);
-
-	status_item_emit_init_steps (item, server->groups->len);
+	status_item_emit_init_steps (item, groups->len);
 	status_item_emit_status (item, _("Updating Group List"));
 
 	/* set the groups column title */
@@ -1224,6 +1248,8 @@ grouplist_update_ui_thread (void* data)
 		default:               title = _("Bug in Code");        break;
 	}
 
+       	sel = grouplist_get_selected_groups ();
+
 	/* update the list */
 	g_static_mutex_lock (&grouplist_ui_mutex);
 	pan_lock ();
@@ -1232,16 +1258,16 @@ grouplist_update_ui_thread (void* data)
 	gtk_clist_set_column_title (clist, 1, title);
 
 	/* add the groups/folders */
-	for (i=server->groups->len-1; i>=0; --i)
+	for (i=groups->len-1; i>=0; --i)
 	{
-		Group * group = GROUP(g_ptr_array_index (server->groups, i));
+		Group * group = GROUP(g_ptr_array_index (groups, i));
 
-		pan_unlock ();
-		status_item_emit_next_step (item);
-		pan_lock ();
+		if (!(i%512)) {
+			pan_unlock ();
+			status_item_emit_set_step (item, groups->len-i);
+			pan_lock ();
+		}
 
-		if (!should_be_in_grouplist (server, group))
-			continue;
 		if (is_nonempty_string(groupname_filter_string) && fnmatch(groupname_filter_string,group->name,PAN_CASEFOLD))
 			continue;
 
@@ -1261,6 +1287,7 @@ grouplist_update_ui_thread (void* data)
 
 	/* cleanup */
 	g_ptr_array_free (sel, TRUE);
+	g_ptr_array_free (groups, TRUE);
 	debug_exit ("grouplist_update_ui_thread");
 }
 
@@ -1273,7 +1300,7 @@ grouplist_update_ui (Server * server)
 	{
 		pthread_t thread;
 
-		pthread_create (&thread, NULL, (void*)grouplist_update_ui_thread, server);
+		pthread_create (&thread, NULL, (void*)grouplist_update_ui_thread, NULL);
 		pthread_detach (thread);
 	}
 
@@ -1311,10 +1338,9 @@ grouplist_update_group_nolock (const Group * group)
 
 		/* icon column */
 		if (group_is_subscribed (group))
-			gtk_clist_set_pixmap (
-				GTK_CLIST(Pan.group_clist), row, 0,
-				group_pixmap->pixmap,
-				group_pixmap->mask);
+			gtk_clist_set_pixmap (GTK_CLIST(Pan.group_clist), row, 0,
+			                      subscribed_pixmap->pixmap,
+			                      subscribed_pixmap->mask);
 		else if (!group_is_folder(group))
 			gtk_clist_set_text (GTK_CLIST(Pan.group_clist),
 					    row, 0, "");
