@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team <pan@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,9 +25,9 @@
 #include <libgnomeui/gnome-app.h>
 #include <libgnomeui/gnome-app-helper.h>
 
-#include "article.h"
-#include "group.h"
-#include "status-item.h"
+#include <pan/base/article.h>
+#include <pan/base/group.h>
+#include <pan/base/status-item.h>
 
 extern GtkWidget * groups_vbox;
 extern GtkWidget * articlelist_ctree;
@@ -47,13 +47,7 @@ enum {
 	MESSAGE_PAGE
 };
 
-void gui_about (void);
-
-void gui_shutdown (void);
-
-void widget_set_font (GtkWidget* w, const char* font_name);
-
-void dialog_draw_centered (GtkWidget *dialog, GtkWidget *window);
+void pan_widget_set_font (GtkWidget* w, const char* font_name);
 
 void gui_set_layout(int);
 
@@ -61,8 +55,6 @@ void gui_construct (const gchar *geometry);
 
 void gui_page_set (int page, GtkWidget *focus_item);
 
-void gui_set_title (const Group*);
-
 void gui_restore_column_widths    (GtkWidget * clist, const gchar * key);
 
 void gui_save_column_widths       (GtkWidget * clist, const gchar * key);
@@ -71,15 +63,8 @@ void gui_restore_window_size      (GtkWidget * window, const gchar * key);
 
 void gui_save_window_size         (GtkWidget * window, const gchar * key);
 
-void gui_select_all (void);
-
-GtkWidget* server_menu_create (void);
-
-
 void gui_set_connecting_flag (gboolean is_connecting);
-void gui_set_connection_size (int size);
 void gui_inc_connection_size (int inc);
-
 void gui_set_queue_size (int running, int total);
 
 extern GnomeUIInfo group_menu [];
