@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team <pan@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,30 +20,31 @@
 
 #include <config.h>
 
-#include <glib.h>
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
-
 #include <ctype.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
 
+#include <langinfo.h>
+
+#include <glib.h>
+
 #include <gmime/gmime-utils.h>
 
-#include "debug.h"
-#include "globals.h"
-#include "gnksa.h"
-#include "grouplist.h"
-#include "log.h"
-#include "message-check.h"
-#include "message-send.h"
-#include "nntp.h"
-#include "prefs.h"
-#include "queue.h"
-#include "task-func-ptr.h"
-#include "smtp.h"
-#include "util.h"
+#include <pan/base/debug.h>
+#include <pan/base/gnksa.h>
+#include <pan/base/log.h>
+#include <pan/base/pan-glib-extensions.h>
+#include <pan/base/pan-i18n.h>
+#include <pan/base/serverlist.h>
+
+#include <pan/message-check-ui.h>
+#include <pan/message-send.h>
+#include <pan/nntp.h>
+#include <pan/prefs.h>
+#include <pan/queue.h>
+#include <pan/task-func-ptr.h>
+#include <pan/smtp.h>
 
 enum MessageSendResult
 {
@@ -85,8 +86,9 @@ build_nntp_header (GString        * postme,
 	g_free (tmp);
 
 	/* Subject: */
-	pan_warn_if_fail (is_nonempty_string (article->subject));
-	tmp = g_mime_utils_8bit_header_encode (article->subject);
+	pch = article_get_subject (article);
+	pan_warn_if_fail (is_nonempty_string (pch));
+	tmp = g_mime_utils_8bit_header_encode (pch);
 	if (subject_append)
 		g_string_sprintfa (postme, "Subject: %s %s\r\n", tmp,
 				subject_append);
@@ -134,9 +136,9 @@ build_nntp_header (GString        * postme,
 
 	/* Message ID: */
 	if (server->gen_msgid) {
-		tmp = g_strdup (article_get_header (article, HEADER_MESSAGE_ID));
+		tmp = g_strdup (article_get_message_id(article));
 		if (force_new_message_id || !is_nonempty_string(tmp))
-			replace_gstr (&tmp, gnksa_generate_message_id(article->author_addr));
+			replace_gstr (&tmp, gnksa_generate_message_id_from_email_addr(article->author_addr));
 		g_string_sprintfa (postme, "Message-ID: %s\r\n", tmp);
 		g_free (tmp);
 	}
@@ -149,9 +151,12 @@ build_nntp_header (GString        * postme,
 	/* User Agent: */
 	g_string_append (postme, "User-Agent: Pan/" VERSION " (Unix)\r\n");
 
-	/* Mime -- this hardcoding will go away soon; this is a stopgap */
+	/* Add mime header, charset type, encoding type */
 	g_string_append (postme, "Mime-Version: 1.0\r\n");
-	g_string_append (postme, "Content-Type: text/plain; charset=ISO-8859-1\r\n");
+	pch = nl_langinfo (CODESET);
+	if (!is_nonempty_string (pch))
+		pch = "ISO-8859-1";
+	g_string_sprintfa (postme, "Content-Type: text/plain; charset=%s\r\n", pch);
 	g_string_append (postme, "Content-Transfer-Encoding: 8bit\r\n");
 
 	/* Extra headers: */
@@ -330,7 +335,7 @@ task_send_describe (const StatusItem * item)
 		fmt = _("Mailing Article ``%s''");
 	else 
 		fmt = _("I have no idea what I'm doing with Article ``%s''");
-	return g_strdup_printf (fmt, article->subject);
+	return g_strdup_printf (fmt, article_get_subject(article));
 }
 
 static void
@@ -379,8 +384,7 @@ task_send_run (TaskFuncPtr* task, gpointer user_data)
 		/* let the user know that something didn't work. */
 		if (text->len != 0) {
 			g_string_append (text, _("Your message was saved in the folder ``pan.sendlater''"));
-			pan_error_dialog (text->str);
-			log_add (LOG_ERROR, text->str);
+			log_add (LOG_ERROR|LOG_URGENT, text->str);
 		}
 
 		g_string_free (text, TRUE);
@@ -391,7 +395,7 @@ task_send_run (TaskFuncPtr* task, gpointer user_data)
 
 		/* save to "Sent".  Note that the call to group_add_article
 		   will also remove the article from pan.sendlater */
-		sent = folder_get_by_name (PAN_SENT);
+		sent = serverlist_get_named_folder (PAN_SENT);
 		group_add_article (sent, article);
 	}
 
@@ -423,7 +427,7 @@ send_article_now (StatusItem           * item,
 
 	/* initialize variables */
        	article = data->article;
-       	subject = article->subject;
+       	subject = article_get_subject (article);
        	mail_to = data->do_mail ? article_get_header (article, PAN_MAIL_TO) : NULL;
        	post_to = data->do_post ? article_get_header (article, HEADER_NEWSGROUPS) : NULL;
 
@@ -445,7 +449,7 @@ send_article_now (StatusItem           * item,
 		/* mail the message */
 		if (!val)
 		{
-			log_add_va (LOG_IMPORTANT, _("E-mail Message \"%s\" sent."), subject);
+			log_add_va (LOG_INFO, _("E-mail Message \"%s\" sent."), subject);
 			data->do_mail = FALSE;
 		}
 		else
@@ -471,11 +475,11 @@ send_article_now (StatusItem           * item,
 			/* let the user know what we're doing */
 			status_item_emit_status_va (STATUS_ITEM(item),
 				_("Posting article \"%s\" part %d of %d."),
-				article->subject, i+1, post_me->len);
+				article_get_subject(article), i+1, post_me->len);
 
 			/* post the part */
 			if (nntp_post(item,sock,a) == TASK_SUCCESS) {
-				log_add_va (LOG_IMPORTANT, _("NNTP Article \"%s\" (%d of %d) posted."),
+				log_add_va (LOG_INFO, _("NNTP Article \"%s\" (%d of %d) posted."),
 					subject, i+1, post_me->len);
 				data->do_post = FALSE;
 			} else {
@@ -537,8 +541,8 @@ queue_article_for_posting (Server     * server,
 		task = TASK(task_func_ptr_new (server,
 		                               task_send_describe,
 		                               task_send_run,
-		                               data,
-		                               task_send_dtor));
+		                               task_send_dtor,
+		                               data, TRUE));
 		task->needs_socket = is_nonempty_string(
 			article_get_header(article,HEADER_NEWSGROUPS));
 		queue_add (task);
@@ -561,7 +565,7 @@ flush_sendlater_articles (void)
 	debug_enter ("flush_sendlater_articles");
 
 	/* find the sendlater folder */
-       	folder = folder_get_by_name (PAN_SENDLATER);
+       	folder = serverlist_get_named_folder (PAN_SENDLATER);
 	g_return_if_fail (folder!=NULL);
 
 	/* load the articles for that group */
@@ -574,10 +578,11 @@ flush_sendlater_articles (void)
 		Server * s;
 	       
 		a = ARTICLE(g_ptr_array_index(articles,i));
-		s = server_get_by_name (article_get_header (a, PAN_SERVER));
+		s = serverlist_get_named_server (article_get_header (a, PAN_SERVER));
 		if (s == NULL) {
-			g_message (_("Article \"%s\" not flagged for any particular server; using current"), a->subject);
-			s = grouplist_get_server ();
+			g_message (_("Article \"%s\" not flagged for any particular server; using current"),
+				article_get_subject (a));
+			s = serverlist_get_active_server ();
 			if (s == NULL)
 				continue;
 		}
