@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team <pan@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,9 +21,9 @@
 #ifndef __MESSAGE_SEND_H__
 #define __MESSAGE_SEND_H__
 
-#include "server.h"
-#include "article.h"
 #include <glib.h>
+#include <pan/base/article.h>
+#include <pan/base/server.h>
 
 gint queue_article_for_posting (Server          * server,
 		                Article         * article,
