@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team <pan@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,24 +29,58 @@
 #include <sys/stat.h>
 #include <unistd.h>
 
-#include "articlelist.h"
-#include "debug.h"
-#include "globals.h"
-#include "gnksa.h"
-#include "grouplist.h"
-#include "gui.h"
-#include "gui-headers.h"
-#include "message-check.h"
-#include "message-send.h"
-#include "message-window.h"
-#include "prefs.h"
-#include "print.h"
-#include "queue.h"
-#include "task-body.h"
-#include "text.h" /* for colors of fg, bg, and quoted text */
-#include "util.h"
-#include "util-wrap.h"
+#include <gnome.h>
 
+#include <pan/base/debug.h>
+#include <pan/base/gnksa.h>
+#include <pan/base/pan-glib-extensions.h>
+#include <pan/base/serverlist.h>
+#include <pan/base/util-file.h>
+#include <pan/base/util-wrap.h>
+
+#include <pan/articlelist.h>
+#include <pan/globals.h>
+#include <pan/grouplist.h>
+#include <pan/gui.h>
+#include <pan/gui-headers.h>
+#include <pan/message-check-ui.h>
+#include <pan/message-send.h>
+#include <pan/message-window.h>
+#include <pan/prefs.h>
+#include <pan/print.h>
+#include <pan/queue.h>
+#include <pan/task-body.h>
+#include <pan/text.h> /* for colors of fg, bg, and quoted text */
+#include <pan/util.h>
+
+typedef struct
+{
+	GtkWidget * window; /* GnomeApp */
+	GtkWidget * appbar; /* GnomeAppBar */
+
+	GtkWidget * read_info_pane;
+
+	GtkWidget * from;
+	GtkWidget * custom_headers_text;
+	GtkWidget * user_agent_tb;
+	GtkWidget * organization;
+	GtkWidget * newsgroups;
+	GtkWidget * users;
+	GtkWidget * subject;
+        GtkWidget * followup_to;
+        GtkWidget * reply_to;
+	GtkWidget * body;
+
+	/* Attachment */
+	GtkWidget * filename;
+	GtkWidget * lines;
+	GtkWidget * lines_per_part;
+	GtkWidget * parts;
+
+	Article * article;
+	MessageWindowType type;
+}
+MessageWindow;
 
 extern GtkTooltips *ttips;
 
@@ -62,6 +96,8 @@ static void message_window_paste_cb        (GtkWidget*, MessageWindow*);
 static void message_window_close           (GtkWidget*, MessageWindow*);
 static void message_window_print           (GtkWidget*, MessageWindow*);
 static void message_window_appbar_toggle   (GtkWidget*, MessageWindow*);
+static void message_reply_external         (GtkWidget*, MessageWindow*);
+
 /* Not ready -- Matt
 static void attachment_file_selector       (GtkWidget*, MessageWindow*);
 static GtkWidget * attachment_page         (MessageWindow*);
@@ -183,6 +219,48 @@ experimental_wrap_handler (GtkText * text, GtkToggleButton * tb)
 ****
 ***/
 
+static gchar*
+article_get_attribution_string (const Article * a)
+{
+	gchar * pch = NULL;
+	gchar * retval = NULL;
+	const gchar * cpch = NULL;
+	GString * gstr = NULL;
+
+	pan_warn_if_fail (is_nonempty_string(attribution_line));
+
+	/* get the unsubstituted string */
+	gstr = g_string_new (attribution_line);
+
+	/* substitutions: message-id */
+	cpch = article_get_message_id (a);
+	pan_g_string_replace (gstr, "%i", cpch);
+
+	/* substitutions: date */
+	pch = a==NULL ? g_strdup("") : rfc822_date_generate(a->date);
+	pan_g_string_replace (gstr, "%d", pch);
+	g_free (pch);
+
+	/* substitutions: author */
+	pch = a==NULL ? g_strdup("") : article_get_author_str (a);
+	pan_g_string_replace (gstr, "%a", pch);
+	g_free (pch);
+
+	/* substitutions: author name */
+	pch = a==NULL ? g_strdup("") : article_get_short_author_str (a);
+	pan_g_string_replace (gstr, "%n", pch);
+	g_free (pch);
+
+	/* cleanup & return the result */
+	retval = gstr->str;
+	g_string_free (gstr, FALSE);
+	return retval;
+}
+
+/***
+****
+***/
+
 static GnomeUIInfo message_edit_file_menu[] =
 {
 	GNOMEUIINFO_ITEM_STOCK (N_("Save Changes"), N_("Save Changes"), 
@@ -401,8 +479,8 @@ append_signature_file (GString * body)
 		gboolean delimiter_exists = (
 			    (!strncmp (sig, "-- \r\n", 5))
 			 || (!strncmp (sig, "-- \n", 4))
-			 || (strstr(sig, "\n-- \r")!=NULL)
-			 || (strstr(sig, "\n-- \n")!=NULL));
+			 || (pan_strstr(sig, "\n-- \r")!=NULL)
+			 || (pan_strstr(sig, "\n-- \n")!=NULL));
 		g_string_append (body, "\n\n");
 		if (!delimiter_exists)
 			g_string_append (body, "\n-- \n");
@@ -455,10 +533,13 @@ create_reply_body (Article * article)
 	}
 	g_free (pch);
 
-	/* wrap what we've got so far */
-	pch = fill_body (out->str, wrap_column);
-	g_string_assign (out, pch);
-	g_free (pch);
+	if (wrap_body_on_reply)
+	{
+		/* wrap what we've got so far */
+		pch = fill_body (out->str, wrap_column);
+		g_string_assign (out, pch);
+		g_free (pch);
+	}
 
 	/* append signature (AFTER the wrap) */
 	append_signature_file (out);
@@ -550,17 +631,26 @@ wrap_cb (GtkWidget *widget, MessageWindow * mw)
 {
 	gchar * body;
 	gchar * new_body;
+	gboolean b;
+	debug_enter ("wrap_cb");
 
 	/* get the current body */
 	pan_lock();
        	body = gtk_editable_get_chars (GTK_EDITABLE(mw->body), 0, -1);
 	pan_unlock ();
 
-	/* fill the text, and update the body */
 	new_body = fill_body (body, wrap_column);
-	g_free (body);
+
+	/* turn off our own wrapping while we fill the body pane */
+	b = experimental_toggle_active;
+	experimental_toggle_active = FALSE;
 	update_body_pane (mw->body, new_body, FALSE);
+	experimental_toggle_active = b;
+
+	/* cleanup */
+	g_free (body);
 	g_free (new_body);
+	debug_exit ("wrap_cb");
 }
 
 /**
@@ -838,7 +928,6 @@ create_post_info_pane (MessageWindow *mw)
 	GtkWidget * w;
 	GtkWidget * main_page;
 	GtkWidget * headers;
-	GtkWidget * eventbox;
 	GtkWidget * notebook;
 
 	pan_lock();
@@ -878,11 +967,7 @@ create_post_info_pane (MessageWindow *mw)
 	/* set the Post To Groups: */
 	w = gtk_label_new ( _("Post To Groups:") );
 	gtk_misc_set_alignment (GTK_MISC(w), 0.0, 0.5);
-	eventbox = gtk_event_box_new ();
-	gtk_container_add (GTK_CONTAINER(eventbox), w);
-	gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), eventbox,
-						  _("Groups to Post this message to"), "");
-	gtk_table_attach (GTK_TABLE(headers), eventbox,
+	gtk_table_attach (GTK_TABLE(headers), w,
 	                  0, 1, 2, 3,
 			  GTK_FILL, GTK_FILL, GNOME_PAD, 0);
 	mw->newsgroups = gtk_entry_new ();
@@ -894,11 +979,7 @@ create_post_info_pane (MessageWindow *mw)
 	/* set the Mail To: */
 	w = gtk_label_new ( _("Mail To:") );
 	gtk_misc_set_alignment (GTK_MISC(w), 0.0, 0.5);
-	eventbox = gtk_event_box_new ();
-	gtk_container_add (GTK_CONTAINER(eventbox), w);
-	gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), eventbox,
-						  _("Users to Mail this message to"), "");
-	gtk_table_attach (GTK_TABLE(headers), eventbox, 0, 1, 3, 4,
+	gtk_table_attach (GTK_TABLE(headers), w, 0, 1, 3, 4,
 			  GTK_FILL, GTK_FILL, GNOME_PAD, 0);
 	mw->users = gtk_entry_new();
 	gtk_table_attach (GTK_TABLE(headers), mw->users,
@@ -951,35 +1032,51 @@ populate_post_info_pane (MessageWindow *mw)
 	**/
 
 	{
-		/* use the Pan defaults */
-		gint i = 0;
-		const gchar * pch = extra_posting_headers;
-		if (is_nonempty_string(pch))
-			gtk_editable_insert_text (
-				GTK_EDITABLE(mw->custom_headers_text),
-				pch, strlen(pch), &i);
+		gchar * pch;
+		const gchar * cpch;
+		GString * gstr = g_string_new (NULL);
 
+		/* start with "X-Comment-To:" header for Fido users. */
+		pch = NULL;
+		if (a!=NULL)
+			pch = article_get_author_str (a);
+		if (!is_nonempty_string(pch))
+			pch = g_strdup ("ALL");
+		g_string_sprintfa (gstr, "X-Comment-To: %s\n", pch);
+		g_free (pch);
 
-		if (a!=NULL && group_is_folder(a->group))
-		{
-			/* walk through all of the article's headers,
-			   and if it's an extra one, and it wasn't covered
-			   in the defaults above, throw it in the list */
-			gint foo = 0;
+		/* add user's defaults */
+		cpch = extra_posting_headers;
+		g_string_append (gstr, cpch);
+
+		/* walk through all of the article's headers.
+		   If it's an extra one & wasn't covered in user's defaults add it */
+		if (a!=NULL && group_is_folder(a->group)) {
 			guint i;
-			GString * str = g_string_new (NULL);
-			GPtrArray * extra_headers = article_get_all_headers (a);
-			for (i=0; i!=extra_headers->len; i+=2) {
-				char* key = (char*)g_ptr_array_index(extra_headers,i);
-				char* val = (char*)g_ptr_array_index(extra_headers,i+1);
+			GPtrArray * h = article_get_all_headers (a);
+			for (i=0; i!=h->len; i+=2) {
+				char* key = (char*)g_ptr_array_index(h,i);
+				char* val = (char*)g_ptr_array_index(h,i+1);
 				gboolean extra = article_header_is_extra (key);
-				if (extra && strstr(pch,key)==NULL)
-					g_string_sprintfa (str, "%s: %s\n", key, val);
+				if (extra && pan_strstr(pch,key)==NULL)
+					g_string_sprintfa (gstr, "%s: %s\n", key, val);
 			}
-			gtk_editable_insert_text (GTK_EDITABLE(mw->custom_headers_text), str->str, str->len, &foo);
-			g_string_free (str, TRUE);
-			g_ptr_array_free (extra_headers, TRUE);
+			g_ptr_array_free (h, TRUE);
 		}
+
+		/* add the headers */
+		if (1) {
+			gint foo = 0;
+			GtkText * text = GTK_TEXT(mw->custom_headers_text);
+			guint len = gtk_text_get_length (text);
+			gtk_text_set_point (text, len);
+			gtk_editable_insert_text (GTK_EDITABLE(text), gstr->str, gstr->len, &foo);
+			if (len)
+				gtk_editable_delete_text (GTK_EDITABLE(text), 0, len);
+		}
+
+		/* cleanup */
+		g_string_free (gstr, TRUE);
 	}
 
 	/**
@@ -1159,6 +1256,8 @@ populate_post_info_pane (MessageWindow *mw)
 			}
 		}
 
+
+
 		/* no explicit newsgroup specified for this post,
 		 * so let's guess where the user might want to post */
 		if (default_newsgroups && is_posting(mw->type))
@@ -1271,7 +1370,7 @@ create_body_pane_nolock (MessageWindow * mw)
 	mw->body = gtk_text_new (NULL, NULL);
 
 	/* set the text widget's properties */
-	widget_set_font (GTK_WIDGET(mw->body), message_body_font);
+	pan_widget_set_font (GTK_WIDGET(mw->body), message_body_font);
 	style = gtk_widget_get_style(mw->body);
 	style->text[0] = text_fg_color;
 	style->base[0] = text_bg_color;
@@ -1287,77 +1386,8 @@ create_body_pane_nolock (MessageWindow * mw)
 }
 
 
-/*---[ update_body_pane ]---------------------------------------------
- * will only be called for messages being "read"
- *--------------------------------------------------------------------*/
 void
-update_body_pane_nolock (GtkWidget    * body_widget,
-                         const char   * body,
-		         gboolean       mute_quotes)
-{
-	GtkText * text = NULL;
-	gchar * tmp;
-
-	/* sanity checks */
-	g_return_if_fail (body_widget!=NULL);
-	g_return_if_fail (GTK_IS_TEXT(body_widget));
-
-	/* mute the quoted text, if desired */
-	tmp = mute_quotes
-		? mute_quoted_text_in_body (body)
-		: g_strdup (body);
-
-	text = GTK_TEXT(body_widget);
-
-	gtk_text_set_point (text, 0);
-	gtk_editable_delete_text (GTK_EDITABLE(body_widget), 0, -1);
-
-	if (is_nonempty_string(tmp))
-	{
-		/* insert the article body */
-		const gchar * march = tmp;
-	       	GString * line = g_string_new (NULL);
-		gtk_text_freeze (text);
-
-		while (get_next_token_g_str(march, '\n', &march, line))
-		{
-			GdkColor * pFG;
-			gboolean is_quote = line->len && line->str[0]=='>';
-
-			/* what color to use? */
-			pFG = is_quote ? &text_quoted_color : &text_fg_color;
-
-			/* strip carriage return */
-			if (line->len!=0 && line->str[line->len-1]=='\r')
-				g_string_erase (line, line->len-1, 1);
-
-			/* add linefeed */
-			if (is_nonempty_string (march))
-				g_string_append_c (line, '\n');
-
-			/* insert this line */
-			gtk_text_insert (text, NULL, pFG, NULL, line->str, line->len);
-		}
-
-		gtk_text_thaw (text);
-		g_string_free (line, TRUE);
-	}
-
-	g_free (tmp);
-}
-
-void
-update_body_pane (GtkWidget    * body_widget,
-                  const char   * body,
-		  gboolean       mute_quotes)
-{
-	pan_lock ();
-	update_body_pane_nolock (body_widget, body, mute_quotes);
-	pan_unlock ();
-}
-
-void
-message_post_window (GtkWidget * widget)
+message_post_window (void)
 {
 	message_window_new (NULL, NNTP_POST);
 }
@@ -1386,27 +1416,8 @@ message_post_window_create (MessageWindow *mw)
 }
 
 
-static int
-message_read_window_cb (gpointer call_object,
-                        gpointer call_arg,
-                        gpointer user_data)
-{
-	const int run_status = GPOINTER_TO_INT (call_arg);
-	TaskBody * task = TASK_BODY(call_object);
-
-	/* entry checks */
-	g_return_val_if_fail (task!=NULL, 0);
-	g_return_val_if_fail (task->article!=NULL, 0);
-
-	/* if it succeeded, open it in a new window */
-	if (run_status == TASK_SUCCESS)
-		message_window_new (task->article, NNTP_READ);
-
-	return 0;
-}
-
 void
-message_read_window (GtkWidget *widget, Article * article)
+message_read_window (Article * article)
 {
 	if (article == NULL)
 		article = get_current_article();
@@ -1429,11 +1440,8 @@ message_read_window (GtkWidget *widget, Article * article)
 		else
 		{
 			/* don't have the body, go download it then pop up the window... */
-			Task * task = TASK(task_body_new_defaults (article));
-			task->high_priority = FALSE;
-			pan_callback_add (task->task_ran_callback,
-			                  message_read_window_cb,
-			                  NULL);
+			Task * task = TASK(task_body_new (article, TRUE, 
+				NNTP_READ));
 			queue_add (task);
 		}
 	}
@@ -1449,7 +1457,7 @@ message_read_window_create (MessageWindow *mw)
 
 	/* create a new window */
 	pan_lock();
-	mw->window = gnome_app_new ("Read", mw->article->subject);
+	mw->window = gnome_app_new ("Read", article_get_subject(mw->article));
 	vbox = gtk_vbox_new (FALSE, 0);
 	gnome_app_set_contents (GNOME_APP (mw->window), vbox);
 	gnome_app_create_menus_with_data (GNOME_APP(mw->window),
@@ -1486,19 +1494,25 @@ message_read_window_create (MessageWindow *mw)
  *
  *--------------------------------------------------------------------*/
 
-void
-message_reply_external (GtkWidget    * widget,
-                        Article      * article)
+static void
+message_reply_external (GtkWidget       * widget,
+                        MessageWindow   * mw)
 {
 	gchar * sub = NULL;
 	gchar * body = NULL;
 	gchar * author = NULL;
+	Article * article;
+
+	g_return_if_fail (mw!=NULL);
+	g_return_if_fail (mw->article!=NULL);
+
+	article = mw->article;
 
 	/* What are we replying to anyway ? */
 	if (article == NULL)
 		article = get_current_article();
 	if (article != NULL) {
-		sub = make_reply_string (article->subject);
+		sub = make_reply_string (article_get_subject(article));
 		author = article_get_author_str (article);
 		body = article_get_body (article);
 	}
@@ -1512,7 +1526,7 @@ message_reply_external (GtkWidget    * widget,
 		pan_g_string_replace (exec, "%s", sub);
 
 		/* If %t is being used, write the body to a temporary file */
-		if ((strstr(external_mailer, "%t")) != NULL) {
+		if ((pan_strstr(external_mailer, "%t")) != NULL) {
 			FILE * f = NULL;
 			gchar * filename = pan_make_temp (&f);
 			if (filename != NULL) {
@@ -1566,7 +1580,7 @@ make_reply_window (Article * article, MessageWindowType type)
 }
 
 void
-message_forward_window (GtkWidget * widget)
+message_forward_window (void)
 {
 	Article * a = get_current_article();
 	if (a != NULL)
@@ -1574,7 +1588,7 @@ message_forward_window (GtkWidget * widget)
 }
 
 void
-message_edit_window (GtkWidget * widget, Article * a)
+message_edit_window (Article * a)
 {
 	if (a == NULL)
 		a = get_current_article();
@@ -1583,7 +1597,7 @@ message_edit_window (GtkWidget * widget, Article * a)
 }
 
 void
-message_reply_window (GtkWidget * widget)
+message_reply_window (void)
 {
 	Article * a = get_current_article();
 	if (a != NULL)
@@ -1591,7 +1605,7 @@ message_reply_window (GtkWidget * widget)
 }
 
 void
-message_followup_reply_window (GtkWidget * widget)
+message_followup_reply_window (void)
 {
 	Article * a = get_current_article();
 	if (a != NULL)
@@ -1600,7 +1614,7 @@ message_followup_reply_window (GtkWidget * widget)
 
 
 void
-message_followup_window (GtkWidget * widget)
+message_followup_window (void)
 {
 	Article * a = get_current_article();
 	if (a != NULL)
@@ -1613,7 +1627,7 @@ message_edit_window_create (MessageWindow *mw)
 	GtkWidget * w;
 
 	pan_lock ();
-	mw->window = gnome_app_new ("Edit", mw->article->subject);
+	mw->window = gnome_app_new ("Edit", article_get_subject(mw->article));
 	gnome_app_create_menus_with_data (GNOME_APP(mw->window),
 	                                  message_edit_main_menu,
 	                                  mw);
@@ -1633,10 +1647,10 @@ message_edit_window_create (MessageWindow *mw)
 	pan_unlock();
 }
 static void
-message_reply_window_create (MessageWindow *mw)
+message_reply_window_create (MessageWindow * mw)
 {
 	GtkWidget * w;
-	gchar * title = make_reply_string (mw->article->subject);
+	gchar * title = make_reply_string (article_get_subject(mw->article));
 
 	pan_lock();
 	mw->window = gnome_app_new ("Reply", title);
@@ -1813,7 +1827,7 @@ message_window_get_group_str (GtkWidget * e)
 		tmp[1] = '\0';
 
 		/* remove empty entries */
-		while ((tmp=strstr(pch,",,")) != NULL)
+		while ((tmp=pan_strstr(pch,",,")) != NULL)
 			g_memmove (tmp, tmp+1, strlen(tmp+1)+1);
 	}
 
@@ -1842,7 +1856,7 @@ message_window_get_reverse_path (const MessageWindow* mw)
 	gchar * address = NULL;
 	gchar * realname = NULL;
 	gchar * retval = NULL;
-	int status = gnksa_do_check_from (from, &address, &realname);
+	int status = gnksa_do_check_from (from, &address, &realname, FALSE);
 	g_free (from);
 	g_free (realname);
 	if (status == GNKSA_OK)
@@ -1870,27 +1884,27 @@ populate_article_from_mw (Article * article, const MessageWindow * mw)
 
 	/* author */
 	pch = get_text_from_editable (mw->from);
-	gnksa_do_check_from (pch, &author_addr, &author_real);
+	gnksa_do_check_from (pch, &author_addr, &author_real, FALSE);
 	if (is_nonempty_string (author_real))
 		gnksa_strip_realname (author_real);
 	article->author_addr = is_nonempty_string (author_addr)
-		? group_chunk_string (article->group, author_addr)
+		? group_chunk_string (article->group, author_addr, TRUE)
 		: NULL;
 	article->author_real = is_nonempty_string (author_real)
-		? group_chunk_string (article->group, author_real)
+		? group_chunk_string (article->group, author_real, TRUE)
 		: NULL;
 	g_free (author_addr);
 	g_free (author_real);
 	g_free (pch);
 
 	/* message-id */
-	pch = gnksa_generate_message_id  (article->author_addr);
-	article_set_header (article, HEADER_MESSAGE_ID, pch, DO_CHUNK);
+	pch = gnksa_generate_message_id_from_email_addr (article->author_addr);
+	article_init_header (article, HEADER_MESSAGE_ID, pch, DO_CHUNK);
 	g_free (pch);
 
 	/* subject */
 	pch = get_text_from_editable (mw->subject);
-	article_set_header (article, HEADER_SUBJECT, pch, DO_CHUNK);
+	article_init_header (article, HEADER_SUBJECT, pch, DO_CHUNK_SHARE);
 	g_free (pch);
 
 	/* references (for posting articles) */
@@ -1898,55 +1912,55 @@ populate_article_from_mw (Article * article, const MessageWindow * mw)
 	    && (mw->article != NULL))
 	{
 		const gchar * refs = article_get_header (mw->article, HEADER_REFERENCES);
-		const gchar * msg_id = article_get_header (mw->article, HEADER_MESSAGE_ID);
+		const gchar * msg_id = article_get_message_id (mw->article);
 		pch = gnksa_generate_references (refs, msg_id);
 		if (pch != NULL) {
-			article_set_header (article, HEADER_REFERENCES, pch, DO_CHUNK);
+			article_init_header (article, HEADER_REFERENCES, pch, DO_CHUNK_SHARE);
 			g_free (pch);
 		}
 	}
 
 	/* organization */
 	pch = message_window_get_organization(mw);
-	article_set_header (article, HEADER_ORGANIZATION, pch, DO_CHUNK);
+	article_init_header (article, HEADER_ORGANIZATION, pch, DO_CHUNK_SHARE);
 	g_free (pch);
 
 	/* followup-to */
 	pch = message_window_get_group_str(mw->followup_to);
-	article_set_header (article, HEADER_FOLLOWUP_TO, pch, DO_CHUNK);
+	article_init_header (article, HEADER_FOLLOWUP_TO, pch, DO_CHUNK_SHARE);
 	g_free (pch);
 
 	/* reply-to */
 	pch = message_window_get_reply_to (mw);
-	article_set_header (article, HEADER_REPLY_TO, pch, DO_CHUNK);
+	article_init_header (article, HEADER_REPLY_TO, pch, DO_CHUNK_SHARE);
 	g_free (pch);
 
 	/* pan-body */
 	pch = get_text_from_editable (mw->body);
-	article_set_header (article, PAN_BODY, pch, DO_CHUNK);
+	article_init_header (article, PAN_BODY, pch, DO_CHUNK);
 	g_free (pch);
 
 	/* newsgroups (for posting articles) */
 	pch = message_window_get_group_str (mw->newsgroups);
-	article_set_header (article, HEADER_NEWSGROUPS, pch, DO_CHUNK);
+	article_init_header (article, HEADER_NEWSGROUPS, pch, DO_CHUNK_SHARE);
 	g_free (pch);
 
 	/* pan-mail-to (for sending mail) */
 	pch = get_text_from_editable (mw->users);
-	article_set_header (article, PAN_MAIL_TO, pch, DO_CHUNK);
+	article_init_header (article, PAN_MAIL_TO, pch, DO_CHUNK_SHARE);
 	g_free (pch);
 
 	/* pan-reverse-path (for sending mail) */
 	pch = message_window_get_reverse_path (mw);
 	if (is_nonempty_string(pch))
-		article_set_header (article, PAN_REVERSE_PATH, pch, DO_CHUNK);
+		article_init_header (article, PAN_REVERSE_PATH, pch, DO_CHUNK_SHARE);
 	g_free (pch);
 
 	/* attribution header (for message-check) */
 	if (mw->article!=NULL) {
 		gchar * attribution = article_get_attribution_string (mw->article);
 		if (is_nonempty_string(attribution))
-			article_set_header (article, PAN_ATTRIBUTION, attribution, DO_CHUNK);
+			article_init_header (article, PAN_ATTRIBUTION, attribution, DO_CHUNK);
 		g_free (attribution);
 	}
 
@@ -1978,7 +1992,7 @@ populate_article_from_mw (Article * article, const MessageWindow * mw)
 			gchar * val = g_strdup (delimit + 1);
 			g_strstrip (key);
 			g_strstrip (val);
-			article_set_header (article, key, val, DO_CHUNK);
+			article_init_header (article, key, val, DO_CHUNK_SHARE);
 			g_free (key);
 			g_free (val);
 		}
@@ -1994,7 +2008,7 @@ build_article_from_window (const MessageWindow * mw)
 	Article * article;
 	debug_enter ("build_article_from_window");
 
-	folder  = folder_get_by_name (PAN_SENDLATER);
+	folder  = serverlist_get_named_folder (PAN_SENDLATER);
 	article = article_new (folder);
 	populate_article_from_mw (article, mw);
 
@@ -2053,7 +2067,7 @@ post (MessageWindow * mw, SendMode mode)
 	GoodnessLevel goodness;
 	debug_enter ("post");
 
-       	server = grouplist_get_server ();
+       	server = serverlist_get_active_server ();
        	errors = g_ptr_array_new ();
 
 	/* Get the article pointer, either through repopulating the
@@ -2071,7 +2085,7 @@ post (MessageWindow * mw, SendMode mode)
 	check_article_and_prompt (article, server, errors, &goodness);
 	if (goodness == OKAY)
 	{
-		Group * sendlater = folder_get_by_name (PAN_SENDLATER);
+		Group * sendlater = serverlist_get_named_folder (PAN_SENDLATER);
 
 		/* gotta ref the articles to make sure that, after sendlater
 		 * takes over memory management of article, it stays alive
@@ -2081,7 +2095,7 @@ post (MessageWindow * mw, SendMode mode)
 		/* store which server to post through, in case the article
 		 * winds up waiting in pan.sendlater */
 		if (!is_nonempty_string(article_get_header (article, PAN_SERVER)))
-			article_set_header (article, PAN_SERVER, server->name, DO_CHUNK);
+			article_set_header (article, PAN_SERVER, server->name, DO_CHUNK_SHARE);
 
 		/* add this article to the sendlater group */
 		group_add_article (sendlater, article);
@@ -2100,7 +2114,7 @@ post (MessageWindow * mw, SendMode mode)
 	pan_g_ptr_array_foreach (errors, (GFunc)g_free, NULL);
 	g_ptr_array_free (errors, TRUE);
 	if (goodness!=OKAY && mw->type!=EDIT_ORIGINAL)
-		group_remove_article (folder_get_by_name(PAN_SENDLATER), article);
+		group_remove_article (serverlist_get_named_folder(PAN_SENDLATER), article);
 
 	debug_exit ("post");
 }
