@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team <pan@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,19 +25,18 @@
 #include <string.h>
 
 #include <glib.h>
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
 
 #include <gmime/gmime-utils.h>
 
-#include "acache.h"
-#include "debug.h"
-#include "gnksa.h"
-#include "log.h"
-#include "nntp.h"
-#include "prefs.h"
-#include "task.h"
-#include "util.h"
+#include <pan/base/acache.h>
+#include <pan/base/debug.h>
+#include <pan/base/log.h>
+#include <pan/base/pan-glib-extensions.h>
+#include <pan/base/pan-i18n.h>
+
+#include <pan/nntp.h>
+#include <pan/prefs.h>
+#include <pan/task.h>
 
 gboolean online_prompt = TRUE;
 
@@ -188,7 +187,7 @@ nntp_disconnect (StatusItem   * status,
 		return val==-1 ? TASK_FAIL : TASK_FAIL_HOPELESS;
 	}
 
-	/* disconnected succcessfully */
+	/* disconnected successfully */
 	debug_exit ("nntp_disconnect");
 	return TASK_SUCCESS;
 }
@@ -230,6 +229,11 @@ nntp_handshake (StatusItem  * status,
 	if (socket->need_auth)
 		val = nntp_authenticate (status, socket);
 
+	/* Achim Settelmeir reports that his local INN doesn't accept GROUP
+	   without MODE READER first */
+	nntp_can_post (NULL, socket, setme_posting_ok);
+
+
 	debug_exit ("nntp_handshake");
 	return val;
 }
@@ -282,32 +286,35 @@ nntp_can_post (StatusItem   * status,
                gboolean     * setme_can_post)
 {
 	gint val;
+	gint retval = 0;
 	const gchar * response = NULL;
 	debug_enter ("nntp_can_post");
 
 	/* entry assertions */
-	g_return_val_if_fail (status!=NULL, TASK_FAIL_HOPELESS);
+	g_return_val_if_fail (sock!=NULL, TASK_FAIL_HOPELESS);
 	g_return_val_if_fail (setme_can_post!=NULL, TASK_FAIL_HOPELESS);
 
 	/* ask the server if we can post or not */
 	val = nntp_command (status, sock, &response, "MODE READER");
-	if (val==200)
-	{
-		*setme_can_post = TRUE;
-		return 0;
-	}
-	if (val==201)
-	{
-		*setme_can_post = FALSE;
-		return 0;
-	}
-	else
-	{
-		*setme_can_post = FALSE;
-		if (status != NULL)
-			status_item_emit_error_va (status, _("MODE READER check failed: %s"),  response);
-		return val;
+	switch (val) {
+		case 200:
+			*setme_can_post = TRUE;
+			retval = 0;
+			break;
+		case 201:
+			*setme_can_post = FALSE;
+			retval = 0;
+			break;
+		default:
+			*setme_can_post = FALSE;
+			if (status != NULL)
+				log_add_va (LOG_ERROR, _("MODE READER check failed: %s"),  response);
+			retval = val;
+			break;
 	}
+
+	debug_exit ("nntp_can_post");
+	return retval;
 }
 
 /**
@@ -415,10 +422,11 @@ nntp_noop (StatusItem    * status,
 **/
 
 static gboolean
-set_header_if_match (Article * a,
-                     const gchar * header,
-                     const gchar * header_key,
-                     const gchar * article_key)
+init_header_if_match (Article       * a,
+                      const gchar   * header,
+                      const gchar   * header_key,
+                      const gchar   * article_key,
+                      HeaderAction    action)
 {
 	gboolean match = FALSE;
 	size_t len;
@@ -434,7 +442,7 @@ set_header_if_match (Article * a,
 	if (!strncmp (header, header_key, len))
 	{
 		gchar * b = g_mime_utils_8bit_header_decode (header+len);
-		article_set_header (a, article_key, b, DO_CHUNK);
+		article_init_header (a, article_key, b, action);
 		g_free (b);
 		match = TRUE;
 	}
@@ -458,22 +466,25 @@ nntp_article_download (StatusItem      * status,
 	GString * buf;
 	debug_enter ("nntp_article_download");
 
-	/* sanity checks */
+	/* sanity clause */
 	g_return_val_if_fail (status!=NULL, TASK_FAIL);
 	g_return_val_if_fail (sock!=NULL, TASK_FAIL);
 	g_return_val_if_fail (group!=NULL, TASK_FAIL);
-	g_return_val_if_fail (article!=NULL, TASK_FAIL);
+	g_return_val_if_fail (article_is_valid(article), TASK_FAIL);
 	g_return_val_if_fail (abort!=NULL, TASK_FAIL);
 
 	/* let the user know what we're doing */
 	if (verbose & NNTP_VERBOSE_TEXT)
-		status_item_emit_status (status, article->subject);
+		status_item_emit_status (status, article_get_subject(article));
 
 	/* request the article from the server */
 	val = nntp_command_va (status, sock, &response, "ARTICLE %lu", article->number);
 	if (val != 220)
 	{
-		gchar * pch = g_strdup_printf (_("Getting article `%s' body failed: %s"), article->subject, response);
+		gchar * pch = g_strdup_printf (_("Getting article `%s' (Message-ID: %s) body failed: %s"),
+		                               article_get_subject(article),
+		                               article_get_message_id(article),
+		                               response);
 		status_item_emit_error (status, pch);
 		g_free (pch);
 
@@ -485,13 +496,15 @@ nntp_article_download (StatusItem      * status,
 
 	/* try to read the article... */
 	getting_headers = TRUE;
-	buf = g_string_new (NULL);
+	buf = g_string_sized_new (80*400); /* prealloc space for ~400 lines */
 	retval = TASK_SUCCESS;
 	if (verbose & NNTP_VERBOSE_INIT_STEPS)
 		status_item_emit_init_steps (status, article->linecount);
+
 	for (;;)
 	{
 		gchar * line;
+		gchar * freeme;
 
 		/* check for end cases: user abort, socket error, and success */
 		if (*abort || pan_socket_getline (sock, &response)) {
@@ -504,7 +517,11 @@ nntp_article_download (StatusItem      * status,
 		}
 
 		/* strip out the \r */
-		line = pan_substitute (response, "\r\n", "\n");
+		freeme = line = pan_substitute (response, "\r\n", "\n");
+
+		/* rfc 977: 2.4.1 */
+		if (line[0]=='.' && line[1]=='.')
+			++line;
 
 		/* save the line */
 		g_string_append (buf, line);
@@ -526,26 +543,34 @@ nntp_article_download (StatusItem      * status,
 
 				/* check for headers to store in Article */
 				if (!matched)
-					matched = set_header_if_match (a, line,
-						"Newsgroups: ",
-						HEADER_NEWSGROUPS);
+					matched = init_header_if_match (a, line,
+					                                "Newsgroups: ",
+					                                HEADER_NEWSGROUPS,
+					                                DO_CHUNK_SHARE);
 				if (!matched)
-					matched = (set_header_if_match (a, line,
-						"Reply-To: ",
-						HEADER_REPLY_TO));
+					matched = init_header_if_match (a, line,
+					                                "Reply-To: ",
+					                                HEADER_REPLY_TO,
+					                                DO_CHUNK_SHARE);
 				if (!matched)
-					matched = (set_header_if_match (a, line,
-						"Followup-To: ",
-						HEADER_FOLLOWUP_TO));
+					matched = init_header_if_match (a, line,
+					                                "Followup-To: ",
+					                                HEADER_FOLLOWUP_TO,
+					                                DO_CHUNK_SHARE);
+				if (!matched)
+					matched = init_header_if_match (a, line,
+					                                "Xref: ",
+					                                HEADER_XREF,
+					                                DO_CHUNK);
 			}
 		}
 
-		g_free (line);
+		g_free (freeme);
 	}
 
 	/* save the message */
 	if (retval == TASK_SUCCESS) {
-        	acache_set_message (article->message_id, buf->str, checkout);
+        	acache_set_message (article_get_message_id(article), buf->str, checkout);
 		articles_remove_flag (&article, 1, STATE_DOWNLOAD_FLAGGED);
 	}
 
@@ -683,7 +708,7 @@ nntp_download_headers (StatusItem       * status,
 		/* create the article data */
 		pch = buffer;
 		a = article_new (group);
-		group->articles_dirty = TRUE;
+		group_set_articles_dirty (group);
 
 		/* article number */
 		a->number = get_next_token_ulong (pch, '\t', &pch);
@@ -693,14 +718,13 @@ nntp_download_headers (StatusItem       * status,
 		replace_gstr (&tmp, g_mime_utils_8bit_header_decode(tmp));
 		if (!is_nonempty_string(tmp))
 			replace_gstr (&tmp, g_strdup(_("No Subject")));
-		article_set_header(a, HEADER_SUBJECT, tmp, DO_CHUNK);
+		article_init_header (a, HEADER_SUBJECT, tmp, DO_CHUNK_SHARE);
 		g_free (tmp);
 
 		/* author */
 		tmp = get_next_token_str (pch, '\t', &pch);
-		replace_gstr (&tmp, g_mime_utils_8bit_header_decode(tmp));
 		if (is_nonempty_string(tmp))
-			article_set_author_from_header (a, tmp);
+			article_init_author_from_header (a, tmp);
 		g_free (tmp);
 
 		/* date */
@@ -712,14 +736,14 @@ nntp_download_headers (StatusItem       * status,
 
 		/* message id */
 		tmp = get_next_token_str (pch, '\t', &pch);
-		if (is_nonempty_string(tmp))
-			article_set_header(a, HEADER_MESSAGE_ID, tmp, DO_CHUNK);
+		if (is_nonempty_string(tmp) && *tmp=='<')
+			article_init_header (a, HEADER_MESSAGE_ID, tmp, DO_CHUNK);
 		g_free (tmp);
 
 		/* references */
 		tmp = get_next_token_str (pch, '\t', &pch);
-		if (is_nonempty_string(tmp))
-			article_set_header(a, HEADER_REFERENCES, tmp, DO_CHUNK);
+		if (is_nonempty_string(tmp) && *tmp=='<')
+			article_init_header (a, HEADER_REFERENCES, tmp, DO_CHUNK_SHARE);
 		g_free (tmp);
 
 		/* unused field: size */
@@ -730,14 +754,18 @@ nntp_download_headers (StatusItem       * status,
 
 		/* crosspost */
 		tmp = get_next_token_str (pch, '\t', &pch);
-		if (tmp==NULL || strncmp (tmp, "Xref:", 5))
+		if (tmp==NULL || strncmp (tmp, "Xref: ", 6))
 			a->crosspost_qty = 1;
-		else {
-			gchar * foo;
+		else
+		{
+			const gchar * foo;
+
 			a->crosspost_qty = 0;
-			for (foo=tmp+5; *foo; ++foo)
+			for (foo=tmp+6; *foo; ++foo)
 				if (*foo==':')
 					++a->crosspost_qty;
+
+			article_init_header (a, HEADER_XREF, tmp+6, DO_CHUNK);
 		}
 		g_free (tmp);
 
@@ -745,9 +773,9 @@ nntp_download_headers (StatusItem       * status,
 		***  Validity Checking
 		**/
 
-		pch = article_get_header (a, HEADER_MESSAGE_ID);
-		if (!is_nonempty_string(pch)) {
-			status_item_emit_error_va (status, _("Corrupt header skipped - no Message Id: %s"), buffer);
+		if (!article_is_valid (a)) {
+			status_item_emit_error_va (status, 
+				_("Corrupt header skipped - no Message Id: %s"), buffer);
 			pan_object_unref (PAN_OBJECT(a));
 			continue;
 		}
@@ -795,14 +823,14 @@ nntp_download_headers (StatusItem       * status,
 		   a single-part binary */
 		if (!parts
 			&& a->linecount>400
-			&& (((strstr(group->name,"binaries")
-					|| strstr(group->name,"fan")
-				        || strstr(group->name,"mag")
-					|| strstr(group->name,"sex")))
-				|| ((strstr(s,".jpg") || strstr(s,".JPG")
-					|| strstr(s,".jpeg") || strstr(s,".JPEG")
-					|| strstr(s,".gif") || strstr(s,".GIF")
-					|| strstr(s,".tiff") || strstr(s,".TIFF")))))
+			&& (((pan_strstr(group->name,"binaries")
+					|| pan_strstr(group->name,"fan")
+				        || pan_strstr(group->name,"mag")
+					|| pan_strstr(group->name,"sex")))
+				|| ((pan_strstr(s,".jpg") || pan_strstr(s,".JPG")
+					|| pan_strstr(s,".jpeg") || pan_strstr(s,".JPEG")
+					|| pan_strstr(s,".gif") || pan_strstr(s,".GIF")
+					|| pan_strstr(s,".tiff") || pan_strstr(s,".TIFF")))))
 			part = parts = 1;
 
 		/* Verify Multipart info */
@@ -891,16 +919,20 @@ nntp_download_bodies (StatusItem       * status,
 
 		status_item_emit_status_va (status, _("Getting %d of %d: \"%s\""),
 		                                    1+*index, articles->len,
-		                                    article->subject);
+		                                    article_get_subject(article));
 
 		/* download if we have to. */
-		pch = article_get_header (article, HEADER_MESSAGE_ID);
-		if (acache_has_message (pch))
+		pch = article_get_message_id (article);
+		if (article_has_body (article))
 		{
 			result = TASK_SUCCESS;
+			status_item_emit_inc_step (status, article->linecount);
 
 			/* since it's cached locally, we can unflag */
 			articles_remove_flag (&article, 1, STATE_DOWNLOAD_FLAGGED);
+
+			if (checkout)
+				acache_file_checkout (&article->message_id, 1);
 		}
 		else
 		{
@@ -930,13 +962,13 @@ nntp_cancel (StatusItem     * status,
 	gchar * author;
 	const gchar * newsgroups;
 	gint retval;
-	const gchar * msg_id = article_get_header (article, HEADER_MESSAGE_ID);
+	const gchar * msg_id = article_get_message_id (article);
 	debug_enter ("nntp_cancel");
 
 	/* sanity checks */
 	g_return_val_if_fail (sock != NULL, TASK_FAIL);
 	g_return_val_if_fail (status != NULL, TASK_FAIL_HOPELESS);
-	g_return_val_if_fail (article != NULL, TASK_FAIL_HOPELESS);
+	g_return_val_if_fail (article_is_valid(article), TASK_FAIL_HOPELESS);
 	g_return_val_if_fail (is_nonempty_string(msg_id), TASK_FAIL_HOPELESS);
 	newsgroups = article_get_header (article, HEADER_NEWSGROUPS);
 	g_return_val_if_fail (is_nonempty_string(newsgroups), TASK_FAIL_HOPELESS);
