@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team <pan@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,25 +21,19 @@
 #include <config.h>
 
 #include <stdlib.h>
-
-#include <glib.h>
-#include <gtk/gtk.h>
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-config.h>
-#include <libgnome/gnome-i18n.h>
-#include <libgnomeui/gnome-dialog.h>
-#include <libgnomeui/gnome-stock.h>
-#include <libgnomeui/gnome-uidefs.h>
-
 #include <stdio.h>
 
-#include "article.h"
-#include "globals.h"
-#include "gui.h"
-#include "prefs.h"
-#include "print.h"
-#include "text.h"
-#include "util.h"
+#include <gnome.h>
+
+#include <pan/base/article.h>
+#include <pan/base/pan-glib-extensions.h>
+
+#include <pan/globals.h>
+#include <pan/gui.h>
+#include <pan/prefs.h>
+#include <pan/print.h>
+#include <pan/text.h>
+#include <pan/util.h>
 
 
 /* private structure for a print dialog */
@@ -116,8 +110,9 @@ print_dialog_print_clicked (PrintDialog * dialog)
 
 	/* execute */
 	if (print_execute (command, text))
-		pan_error_dialog (
-			"Printing of message \"%s\" failed!", article->subject);
+		pan_error_dialog_parented (dialog->window,
+			"Printing of message \"%s\" failed!",
+			article_get_subject(article));
 	else
 		gnome_config_set_string ("/Pan/Print/print_command", command);
 
@@ -168,7 +163,7 @@ print_dialog (Article * article)
 	/* dialog object */
 	dialog = g_new0 (PrintDialog, 1);
 	dialog->article = article;
-	dialog->window = gnome_dialog_new (_("Print"),
+	dialog->window = gnome_dialog_new (_("Pan: Print"),
 					   _("Print"),
 					   GNOME_STOCK_BUTTON_CANCEL, NULL);
 
