@@ -1,6 +1,8 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
+ * Author: Charles Kerr <charles@rebelbase.com>
+ *
+ * Copyright (C) 2000, 2001  Pan Development Team <pan@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,8 +18,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  * 
- *
- *
  * A note on implementation
  * The queue defers all public tasks (like insert, remove, etc.)
  * to the queue thread.  This ensures that all tasks are handled
@@ -38,16 +38,18 @@
 #include <errno.h>
 
 #include <glib.h>
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
 
-#include "debug.h"
-#include "gui.h"
-#include "log.h"
-#include "nntp.h"
-#include "prefs.h"
-#include "queue.h"
-#include "util.h"
+#include <pan/base/debug.h>
+#include <pan/base/log.h>
+#include <pan/base/pan-glib-extensions.h>
+#include <pan/base/pan-i18n.h>
+#include <pan/base/server.h>
+
+#include <pan/gui.h>
+#include <pan/prefs.h>
+#include <pan/nntp.h>
+#include <pan/queue.h>
+#include <pan/server-ui.h>
 
 /*********************
 **********************  Defines / Enumerated types
@@ -62,7 +64,8 @@ typedef enum
 	TODO_TASK_ABORT,
 	TODO_SET_MAX_SOCKET_QTY,
 	TODO_SET_MAX_TRIES,
-	TODO_SHUTDOWN
+	TODO_SHUTDOWN,
+	TODO_PAUSED
 }
 QueueTodoAction;
 
@@ -176,6 +179,8 @@ static int max_tries = 5;
 
 static int running_tasks = 0;
 
+static gboolean paused = FALSE;
+
 /*********************
 **********************  BEGINNING OF SOURCE
 *********************/
@@ -187,14 +192,27 @@ static int running_tasks = 0;
 void
 queue_wakeup (void)
 {
+	debug_enter ("queue_wakeup");
         pthread_mutex_lock (&queue_lock);
         pthread_cond_broadcast (&qcond);
         pthread_mutex_unlock (&queue_lock);
+	debug_exit ("queue_wakeup");
+}
+
+static gint
+server_online_status_changed_cb (gpointer obj, gpointer arg, gpointer data)
+{
+	debug_enter ("server_online_status_changed_cb");
+	queue_wakeup ();
+	debug_exit ("server_online_status_changed_cb");
+	return 0;
 }
 
 void
 queue_init (int socket_qty, int tries)
 {
+	debug_enter ("queue_init");
+
 	task_to_status = g_hash_table_new (g_direct_hash, g_direct_equal);
 
 	queue_task_added = pan_callback_new ();
@@ -206,14 +224,46 @@ queue_init (int socket_qty, int tries)
 	pthread_mutex_init (&queue_lock, NULL);
 	pthread_mutex_init (&sock_lock, NULL);
 
+	pan_callback_add (server_get_online_status_changed_callback(),
+	                  server_online_status_changed_cb, NULL);
+
 	max_tries = tries;
 	max_socket_qty = socket_qty;
 
 	/* fire up the queue thread */
 	pthread_create (&queue_thread, NULL, (void*)queue_mainloop, NULL);
 	pthread_detach (queue_thread);
+
+	debug_exit ("queue_init");
 }
 
+
+gboolean
+queue_is_paused (void)
+{
+	return paused;
+}
+void
+queue_set_paused (gboolean paused)
+{
+	queue_new_todo (TODO_PAUSED, NULL, paused, -1);
+}
+static void
+real_queue_set_paused (gint p)
+{
+	if ((paused = p))
+	{
+		GSList * l;
+		for (l=sockets; l!=NULL; l=l->next)
+		{
+			SocketInfo * i = (SocketInfo*)l->data;
+			task_hint_abort (i->task);
+			queue_set_task_status (i->task, QUEUE_TASK_STATUS_ABORTING);
+		}
+	}
+}
+
+
 void
 queue_shutdown (gboolean save_tasks)
 {
@@ -226,10 +276,16 @@ real_queue_shutdown (int save_tasks)
 	GSList * l;
 	GSList * sockets_tmp;
 
+	/* stop listening to events */
+	pan_callback_remove (server_get_online_status_changed_callback(),
+	                     server_online_status_changed_cb, NULL);
+
 	/* clear the status hashtable */
 	g_hash_table_destroy (task_to_status);
 	task_to_status = NULL;
 
+
+
 	/* optionally save the tasks */
 	if (save_tasks)
 	{
@@ -318,13 +374,23 @@ static void
 real_queue_task_abort (Task* task)
 {
 	QueueTaskStatus status = queue_get_task_status (task);
+
 	if (status==QUEUE_TASK_STATUS_RUNNING)
 	{
-		task_hint_abort (TASK(task), TRUE);
+		/* ask a running task to abort */
+		task_hint_abort (TASK(task));
 		queue_set_task_status (task, QUEUE_TASK_STATUS_ABORTING);
 	}
+	else if (task->hint_abort && paused)
+	{
+		/* the task's been stopped because the queue is paused */
+		task->tries = 0;
+		task->hint_abort = FALSE;
+		queue_set_task_status (task, QUEUE_TASK_STATUS_QUEUED);
+	}
 	else
 	{
+		/* the task has failed */
 		task->tries = 0;
 		task->hint_abort = FALSE;
 		queue_set_task_status (task, QUEUE_TASK_STATUS_FAILED);
@@ -345,17 +411,12 @@ queue_task_insert (Task* task, int index)
 static void
 real_queue_task_insert (Task* task, int index)
 {
-	debug4 (DEBUG_QUEUE,
-		"%s:%d queue_task_insert %p to pos %d",
-		__FILE__, __LINE__, task, index);
+	debug2 (DEBUG_QUEUE, "queue_task_insert %p to pos %d", task, index);
 
 	g_return_if_fail (task!=NULL);
 	g_return_if_fail (index>=-1);
 	g_return_if_fail (index<=(int)queue_get_length());
 
-	pan_object_ref (PAN_OBJECT(task));
-	pan_object_sink (PAN_OBJECT(task));
-
 	/* add to the list */
 	pthread_mutex_lock (&qlock);
 	queue_list = g_slist_insert (queue_list, task, index);
@@ -365,16 +426,12 @@ real_queue_task_insert (Task* task, int index)
 	gui_set_queue_size (running_tasks, (int)g_slist_length(queue_list));
 
 	/* call callbacks */
-	debug3 (DEBUG_QUEUE,
-		"%s:%d queue size is now %d, calling callbacks",
-		__FILE__, __LINE__, queue_get_length());
+	debug1 (DEBUG_QUEUE, "queue size is now %d, calling callbacks", queue_get_length());
 	pan_callback_call (queue_task_added, task, GINT_TO_POINTER(index));
 
 	queue_wakeup ();
 	
-	debug4 (DEBUG_QUEUE,
-		"%s:%d queue_task_insert %p to pos %d done",
-		__FILE__, __LINE__, task, index);
+	debug2 (DEBUG_QUEUE, "queue_task_insert %p to pos %d done", task, index);
 }
 
 /**
@@ -389,15 +446,13 @@ queue_task_remove (Task* task)
 static void
 real_queue_task_remove (Task* task)
 {
-	debug3 (DEBUG_QUEUE,
-	        "%s:%d queue_task_remove %p",
-	        __FILE__, __LINE__, task);
+	debug1 (DEBUG_QUEUE, "queue_task_remove %p", task);
 
 	g_return_if_fail (task!=NULL);
 
 	if (task->thread_id != 0) /* if it's running, send an abort message */
 	{
-		task_hint_abort (task, TRUE);
+		task_hint_abort (task);
 		queue_set_task_status (task, QUEUE_TASK_STATUS_ABORTING);
 	}
 	else /* not running; simple cleanup. queue_run_thread() zeroes out thread_id */
@@ -422,9 +477,7 @@ real_queue_task_remove (Task* task)
 			queue_wakeup ();
 	}
 
-	debug3 (DEBUG_QUEUE,
-	        "%s:%d queue_task_remove %p done",
-	        __FILE__, __LINE__, task);
+	debug1 (DEBUG_QUEUE, "queue_task_remove %p done", task);
 }
 
 /**
@@ -442,10 +495,9 @@ real_queue_task_move (Task* task, int index)
 	int old_index;
 	int indices [2];
 
-	debug4 (DEBUG_QUEUE,
-	        "%s:%d queue_task_move %p to %d",
-	        __FILE__, __LINE__, task, index);
+	debug2 (DEBUG_QUEUE, "queue_task_move %p to %d", task, index);
 
+	/* sanity clause */
 	g_return_if_fail (task!=NULL);
 	g_return_if_fail (index>=-1);
 	g_return_if_fail (index<=(int)queue_get_length());
@@ -463,15 +515,13 @@ real_queue_task_move (Task* task, int index)
 
 	pthread_mutex_unlock (&qlock);
 
-	debug5 (DEBUG_QUEUE,
-                "%s:%d queue_task_move %p calling callbacks; "
+	debug3 (DEBUG_QUEUE,
+                "queue_task_move %p calling callbacks; "
                 "old index is %d, new index is %d",
-                __FILE__, __LINE__, task, old_index, index);
+                task, old_index, index);
 	pan_callback_call (queue_task_moved, task, indices);
 
-	debug3 (DEBUG_QUEUE,
-	        "%s:%d queue_task_move %p done",
-	        __FILE__, __LINE__, task);
+	debug1 (DEBUG_QUEUE, "queue_task_move %p done", task);
 }
 
 /**
@@ -564,8 +614,8 @@ queue_new_todo (QueueTodoAction action, Task* task, int i1, int i2)
 
 	pthread_mutex_lock (&queue_lock);
 	todo_list = g_slist_append (todo_list, todo);
-	debug5 (DEBUG_QUEUE, "%s:%d todo queue now has %d tasks (new task type: %d, int 1: %d)",
-		__FILE__, __LINE__, (int)g_slist_length(todo_list), action, i1);
+	debug3 (DEBUG_QUEUE, "todo queue now has %d tasks (new task type: %d, int 1: %d)",
+		(int)g_slist_length(todo_list), action, i1);
 	pthread_mutex_unlock (&queue_lock);
 
 	queue_wakeup ();
@@ -597,10 +647,7 @@ queue_run_thread (void* vp)
 	Task* task = TASK(vp);
 	gchar* desc = status_item_describe (STATUS_ITEM(task));
 	gboolean aborted;
-
-	debug4 (DEBUG_QUEUE,
-		"%s:%d queue_run_thread: task %p [%s]",
-		__FILE__, __LINE__, task, desc);
+	debug2 (DEBUG_QUEUE, "queue_run_thread: task %p [%s]", task, desc);
 
 	/* run */
 	status_item_set_active (STATUS_ITEM(task), TRUE);
@@ -609,9 +656,7 @@ queue_run_thread (void* vp)
 	gui_set_queue_size (++running_tasks, (int)g_slist_length(queue_list));
 	result = task_run (task);
 	gui_set_queue_size (--running_tasks, (int)g_slist_length(queue_list));
-	debug5 (DEBUG_QUEUE,
-		"%s:%d queue task[%p] (%s) returned: %d",
-		__FILE__, __LINE__, task,desc,result);
+	debug3 (DEBUG_QUEUE, "queue task[%p] (%s) returned: %d", task, desc, result);
 	g_free(desc);
 
 	/* clean up gui */
@@ -646,10 +691,7 @@ queue_run_thread (void* vp)
 		queue_set_task_status (task, QUEUE_TASK_STATUS_FAILED);
 	}
 
-	debug3 (DEBUG_QUEUE,
-		"%s:%d queue_run_thread: task %p done",
-		__FILE__, __LINE__, vp);
-
+	debug1 (DEBUG_QUEUE, "queue_run_thread: task %p done", vp);
 	pthread_exit (NULL);
 }
 
@@ -658,10 +700,7 @@ static void
 queue_run (Task* task)
 {
 	g_return_if_fail (task!=NULL);
-
-	debug4 (DEBUG_QUEUE,
-		"%s:%d starting a thread to run task %p, sock %p",
-		__FILE__, __LINE__, task, task->sock );
+	debug2 (DEBUG_QUEUE, "starting a thread to run task %p, sock %p", task, task->sock);
 
 	queue_set_task_status (task, QUEUE_TASK_STATUS_RUNNING);
 
@@ -688,10 +727,7 @@ queue_get_socket_for_task (Task *task)
 	gboolean reader_socket_exists = FALSE;
 	Server * server = task->server;
 	int socket_qty;
-
-	debug3 (DEBUG_QUEUE,
-		"%s:%d queue_get_socket_for_task %p",
-		__FILE__, __LINE__, task );
+	debug_enter ("queue_get_socket_for_task");
 
 	/* if we're not online, then try to go online, or not. */
 	if (!server->online)
@@ -740,10 +776,9 @@ queue_get_socket_for_task (Task *task)
 		si->task = task;
 		task->sock = si->socket;
 
-		debug5 (DEBUG_QUEUE,
-			"%s:%d queue_get_socket_for_task %p "
+		debug3 (DEBUG_QUEUE,
+			"queue_get_socket_for_task %p "
 			"reusing socket #%d (%p)",
-			__FILE__, __LINE__, 
 			task, g_slist_index(sockets, si), si->socket);
 
 		retval = 0;
@@ -754,10 +789,7 @@ queue_get_socket_for_task (Task *task)
 	{
 		const int current_connections = socket_qty;
 		int slots_open = server->max_connections - current_connections;
-
-		debug3 (DEBUG_QUEUE,
-			"%s:%d queue_get_socket_for_task %p "
-			"no idle sockets", __FILE__, __LINE__, task);
+		debug1 (DEBUG_QUEUE, "queue_get_socket_for_task %p no idle sockets", task);
 
 		/* leave an article reader socket free */
 		if (!reader_socket_exists
@@ -769,10 +801,9 @@ queue_get_socket_for_task (Task *task)
 
 		if (slots_open <= 0)
 		{
-			debug4 (DEBUG_QUEUE,
-				"%s:%d we've already got %d connections "
+			debug2 (DEBUG_QUEUE,
+				"we've already got %d connections "
 				"and %d reader connections",
-				__FILE__, __LINE__,
 				current_connections,
 				(reader_socket_exists?1:0) );
 
@@ -784,20 +815,13 @@ queue_get_socket_for_task (Task *task)
 
 			/* open a new socket.. */
 			PanSocket *sock = NULL;
-			debug3 (DEBUG_QUEUE,
-				"%s:%d making a new socket to %s",
-				__FILE__, __LINE__, 
-				server->address);
+			debug1 (DEBUG_QUEUE, "making a new socket to %s", server->address);
 
 			gui_set_connecting_flag (TRUE);
 			sock = pan_socket_new (server->address, server->port);
 			pan_socket_set_nntp_auth (sock, server->need_auth, server->username, server->password); 
-			pan_object_ref (PAN_OBJECT(sock));
-			pan_object_sink (PAN_OBJECT(sock));
 
-			debug3 (DEBUG_QUEUE,
-				"%s:%d new socket %p",
-				__FILE__, __LINE__, sock);
+			debug1 (DEBUG_QUEUE, "new socket %p", sock);
 			if (sock->sockfd < 0)
 				errmsg = "Unable to connect to server";
 			else /* handshake */
@@ -816,9 +840,7 @@ queue_get_socket_for_task (Task *task)
 					pan_object_unref (PAN_OBJECT(sock));
 				sock = NULL;
 
-				debug3 (DEBUG_QUEUE,
-					"%s:%d %p can't log in!",
-					__FILE__, __LINE__, sock);
+				debug1 (DEBUG_QUEUE, "%p can't log in!", sock);
 
 				retval = -2;
 			}
@@ -826,9 +848,7 @@ queue_get_socket_for_task (Task *task)
 			{
 				SocketInfo * si = NULL;
 
-				debug3 (DEBUG_QUEUE,
-					"%s:%d %p logged in",
-					__FILE__, __LINE__, sock);
+				debug1 (DEBUG_QUEUE, "%p logged in", sock);
 
 				/* looks like we got a new socket okay... */
 				si = g_new0 (SocketInfo, 1);
@@ -847,10 +867,7 @@ queue_get_socket_for_task (Task *task)
 
 	pthread_mutex_unlock (&sock_lock);
 
-	debug3 (DEBUG_QUEUE,
-		"%s:%d queue_get_socket_for_item was %ssuccessful",
-		__FILE__, __LINE__, (retval?"not ":""));
-
+	debug_exit ("queue_get_socket_for_item");
 	return retval;
 }
 
@@ -862,11 +879,7 @@ static void
 queue_run_what_we_can (void)
 {
 	GSList *l = NULL;
-
-	debug3 (DEBUG_QUEUE,
-		"%s:%d queue_run_what_we_can (%d in queue)",
-		__FILE__, __LINE__,
-		queue_get_length());
+	debug_enter ("queue_run_what_we_can");
 
 	/* walk through the queue and see if
 	   any tasks are waiting to be run. */
@@ -878,33 +891,31 @@ queue_run_what_we_can (void)
 		/* if it's not waiting to be run, leave it alone */
 		if (queue_get_task_status(task) != QUEUE_TASK_STATUS_QUEUED)
 		{
-			debug2 (DEBUG_QUEUE,
-				"%s:%d task isn't waiting to be run",
-				__FILE__, __LINE__);
+			debug0 (DEBUG_QUEUE, "task isn't waiting to be run");
 
 		}
 
 		/* if it doesn't need a socket, no problemo! */
 		else if (!task->needs_socket)
 		{
-			debug3 (DEBUG_QUEUE,
-				"%s:%d task %d needs no socket; running it now",
-				__FILE__, __LINE__, task);
+			debug1 (DEBUG_QUEUE,
+				"task %d needs no socket; running it now",
+				task);
 
 			queue_run (task);
 		}
 
 		/* try to get a socket... */
-		else if (max_socket_qty>0)
+		else if (!paused && max_socket_qty>0)
 		{
 			int i;
 			queue_set_task_status (task, QUEUE_TASK_STATUS_CONNECTING);
 			i = queue_get_socket_for_task (task);
 			if (i==0)
 			{
-				debug3 (DEBUG_QUEUE,
-					"%s:%d task %d got a socket!",
-					__FILE__, __LINE__, task);
+				debug1 (DEBUG_QUEUE,
+					"task %d got a socket!",
+					task);
 
 				queue_run (task);
 			}
@@ -934,9 +945,7 @@ queue_run_what_we_can (void)
 		}
 	}
 
-	debug2 (DEBUG_QUEUE,
-		"%s:%d queue_run_what_we_can done",
-		__FILE__, __LINE__);
+	debug_exit ("queue_run_what_we_can done");
 }
 
 /**
@@ -946,9 +955,7 @@ queue_run_what_we_can (void)
 static void
 socket_cleanup (Task* task)
 {
-	debug4 (DEBUG_QUEUE,
-		"%s:%d socket_cleanup for task %p (sock %p)",
-		__FILE__, __LINE__, task, task?task->sock:NULL);
+	debug_enter ("socket_cleanup");
 
 	if (task && task->sock)
 	{
@@ -985,9 +992,9 @@ socket_cleanup (Task* task)
 		       	/* if socket's okay, reuse it */
 			info->task = NULL;
 
-			debug3 (DEBUG_QUEUE,
-				"%s:%d socket_cleanup: recycling socket %p",
-				__FILE__, __LINE__, socket);
+			debug1 (DEBUG_QUEUE,
+				"socket_cleanup: recycling socket %p",
+				socket);
 		}
 		else
 		{
@@ -997,15 +1004,17 @@ socket_cleanup (Task* task)
 			sockets = g_slist_remove (sockets, info);
 			g_free (info);
 
-			debug3 (DEBUG_QUEUE,
-				"%s:%d socket_cleanup: throwing away socket %p",
-				__FILE__, __LINE__, socket);
+			debug1 (DEBUG_QUEUE,
+				"socket_cleanup: throwing away socket %p",
+				socket);
 		}
 
 		task->sock = NULL;
 
 		pthread_mutex_unlock (&sock_lock);
 	}
+
+	debug_exit ("socket_cleanup");
 }
 
 static gchar*
@@ -1082,9 +1091,9 @@ sockets_upkeep (void)
 		}
 	}
 
-	debug3 (DEBUG_QUEUE,
-		"%s:%d sockets_upkeep: culling %d sockets",
-		__FILE__, __LINE__, g_slist_length(cull));
+	debug1 (DEBUG_QUEUE,
+		"sockets_upkeep: culling %d sockets",
+		g_slist_length(cull));
 
 	for (l=cull; l!=NULL; l=l->next)
 	{
@@ -1148,8 +1157,7 @@ queue_do_todo (void)
 				real_queue_task_move (a->task, a->index_1);
 				break;
 			case TODO_REQUEUE:
-				real_queue_task_requeue_failed (
-					a->task, a->index_1);
+				real_queue_task_requeue_failed (a->task, a->index_1);
 				break;
 			case TODO_TASK_ABORT:
 				real_queue_task_abort (a->task);
@@ -1164,6 +1172,9 @@ queue_do_todo (void)
 				real_queue_shutdown (a->index_1);
 				l = NULL;
 				break;
+			case TODO_PAUSED:
+				real_queue_set_paused (a->index_1);
+				break;
 			default:
 				pan_warn_if_reached();
 				break;
