@@ -1,6 +1,8 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
+ * Author: Charles Kerr <charles@rebelbase.com>
+ *
+ * Copyright (C) 2000, 2001  Pan Development Team <pan@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,7 +25,7 @@
 
 #include <glib.h>
 
-#include "task.h"
+#include <pan/task.h>
 
 
 
@@ -33,6 +35,10 @@ extern void queue_add (Task*);
 
 extern void queue_shutdown (gboolean save_tasks);
 
+extern void queue_set_paused (gboolean paused);
+
+extern gboolean queue_is_paused (void);
+
 
 /**
 ***  This rest of this file is for the queue manager.
