@@ -1,6 +1,8 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
+ * Author: Charles Kerr <charles@rebelbase.com>
+ *
+ * Copyright (C) 2000, 2001  Pan Development Team <pan@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,10 +24,9 @@
 #define __SAVE_H__
 
 #include <glib.h>
-#include "server.h"
-#include "group.h"
 
-extern void save_current_article (void);
+#include <pan/base/article.h>
+#include <pan/base/group.h>
 
 /**
  * @param Group* the group to use when fetching the articles
@@ -37,4 +38,9 @@ extern void save_current_article (void);
  */
 extern void save_attachment_as (Group*, GSList*);
 
+extern void save_article (Article * a);
+
+extern void save_current_article (void);
+
+
 #endif /* __QUEUE_ITEM_BODY__H__ */
