@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team <pan@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,16 +28,17 @@
 #include <string.h>
 
 #include <glib.h>
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
 
-#include "article.h"
-#include "debug.h"
-#include "log.h"
-#include "nntp.h"
-#include "queue.h"
-#include "task-bodies.h"
-#include "util.h"
+#include <pan/base/acache.h>
+#include <pan/base/article.h>
+#include <pan/base/debug.h>
+#include <pan/base/log.h>
+#include <pan/base/pan-glib-extensions.h>
+#include <pan/base/pan-i18n.h>
+
+#include <pan/nntp.h>
+#include <pan/queue.h>
+#include <pan/task-bodies.h>
 
 /*********************
 **********************  Defines / Enumerated types
@@ -87,6 +88,14 @@ task_bodies_destructor (PanObject *o)
 
 	/* task-bodies dtor: unref articles + group */
  	task = TASK_BODIES(o);
+	if (task->articles->len) {
+		guint i;
+		const gchar ** message_ids = g_new0 (const gchar*, task->articles->len);
+		for (i=0; i<task->articles->len; ++i)
+			message_ids[i] = article_get_message_id (ARTICLE(g_ptr_array_index(task->articles,i)));
+		acache_file_checkin (message_ids, task->articles->len);
+		g_free (message_ids);
+	}
 	pan_g_ptr_array_foreach (task->articles, (GFunc)pan_object_unref, NULL);
 	group_unref_articles (task->group, STATUS_ITEM(task));
 
@@ -109,7 +118,9 @@ PanObject*
 task_bodies_new (Group             * group,
                  const GPtrArray   * articles)
 {
+	gint i;
 	TaskBodies* bodies = NULL;
+	gboolean needs_socket;
 	debug_enter ("task_bodies_new");
 
 	/* sanity clause */
@@ -117,6 +128,22 @@ task_bodies_new (Group             * group,
 	g_return_val_if_fail (articles!=NULL, NULL);
 	g_return_val_if_fail (articles->len!=0, NULL);
 
+	/* check out all the articles */
+	if (articles->len) {
+		guint i;
+		const gchar ** message_ids = g_new0 (const gchar*, articles->len);
+		for (i=0; i<articles->len; ++i)
+			message_ids[i] = article_get_message_id (ARTICLE(g_ptr_array_index(articles,i)));
+		acache_file_checkout (message_ids, articles->len);
+		g_free (message_ids);
+	}
+
+	/* do we need a connection? */
+	needs_socket = FALSE;
+	for (i=0; i<articles->len && !needs_socket; ++i)
+		if (!article_has_body(ARTICLE(g_ptr_array_index(articles,i))))
+			needs_socket = TRUE;
+
 	/* create the object */
 	group_ref_articles (group, NULL);
 	pan_g_ptr_array_foreach ((GPtrArray*)articles, (GFunc)pan_object_ref, NULL);
@@ -127,7 +154,8 @@ task_bodies_new (Group             * group,
 	task_constructor (TASK(bodies), task_bodies_destructor,
 	                                task_bodies_describe,
 	                                task_bodies_run,
-	                                group->server, TRUE, TRUE);
+	                                group->server, TRUE,
+	                                needs_socket);
 	TASK(bodies)->gets_bodies = TRUE;
 
 	/* initialize the task-bodies */
@@ -164,20 +192,27 @@ task_bodies_run (Task* task)
 	gint steps = (int)TASK_BODIES(task)->articles->len;
 	debug_enter ("task_bodies_run");
 
-	status_item_emit_init_steps (STATUS_ITEM(task), steps);
+	if (!task->needs_socket)
+	{
+		retval = TASK_SUCCESS;
+	}
+	else
+	{
+		status_item_emit_init_steps (STATUS_ITEM(task), steps);
 
-	retval = nntp_download_bodies (STATUS_ITEM(task),
-	                               TASK_BODIES(task)->group,
-                                       task->sock,
-                                       &task->hint_abort,
-                                       TASK_BODIES(task)->articles,
-                                       &TASK_BODIES(task)->working_index,
-                                       FALSE, FALSE);
+		retval = nntp_download_bodies (STATUS_ITEM(task),
+					       TASK_BODIES(task)->group,
+					       task->sock,
+					       &task->hint_abort,
+					       TASK_BODIES(task)->articles,
+					       &TASK_BODIES(task)->working_index,
+					       FALSE, FALSE);
 
-	if (retval == TASK_SUCCESS) {
-		log_add_va (LOG_INFO, ("Fetched %u bodies from `%s'"),
-			steps,
-			group_get_readable_name(TASK_BODIES(task)->group));
+		if (retval == TASK_SUCCESS) {
+			log_add_va (LOG_INFO, ("Fetched %u bodies from `%s'"),
+				steps,
+				group_get_readable_name(TASK_BODIES(task)->group));
+		}
 	}
 
 	debug_exit ("task_bodies_run");
