@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team <pan@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,20 +28,19 @@
 #include <string.h>
 
 #include <glib.h>
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
 
-#include <pan/acache.h>
-#include <pan/article.h>
-#include <pan/debug.h>
-#include <pan/globals.h>
-#include <pan/log.h>
+#include <pan/base/acache.h>
+#include <pan/base/article.h>
+#include <pan/base/debug.h>
+#include <pan/base/log.h>
+#include <pan/base/pan-i18n.h>
+#include <pan/base/pan-glib-extensions.h>
+
 #include <pan/message-window.h>
 #include <pan/nntp.h>
 #include <pan/queue.h>
 #include <pan/task-body.h>
 #include <pan/text.h>
-#include <pan/util.h>
 
 /*********************
 **********************  Defines / Enumerated types
@@ -128,7 +127,7 @@ task_body_new (Article            * a,
 	debug_enter ("task_body_new");
 
  	is_folder = group_is_folder (a->group);
-	message_id = a->message_id;
+	message_id = article_get_message_id (a);
 
 	/* sanity clause */
 	g_return_val_if_fail (a!=NULL, NULL);
@@ -187,13 +186,11 @@ task_body_describe (const StatusItem * status)
 
 	/* sanity clause */
 	g_return_val_if_fail (status!=NULL, NULL);
-	g_return_val_if_fail (TASK_BODY(status)->article!=NULL, NULL);
+	g_return_val_if_fail (article_is_valid(TASK_BODY(status)->article), NULL);
 	g_return_val_if_fail (TASK_BODY(status)->article->group!=NULL, NULL);
-	g_return_val_if_fail (TASK_BODY(status)->article->message_id!=NULL, NULL);
-	g_return_val_if_fail (TASK_BODY(status)->article->subject!=NULL, NULL);
 
-	message_id = TASK_BODY(status)->article->message_id;
-	subject = TASK_BODY(status)->article->subject;
+	message_id = article_get_message_id (TASK_BODY(status)->article);
+	subject = article_get_subject (TASK_BODY(status)->article);
 	group = TASK_BODY(status)->article->group;
 
 	retval = NULL;
@@ -240,7 +237,7 @@ task_body_run (Task * task)
 	group = body->article->group;
 	article = body->article;
 	abort = &task->hint_abort;
-	message_id =article_get_header(article,HEADER_MESSAGE_ID);
+	message_id = article_get_message_id (article);
 	
 	/* sanity clause */
 	g_return_val_if_fail (server!=NULL, TASK_FAIL_HOPELESS);
@@ -292,11 +289,11 @@ task_body_run (Task * task)
 		{
 			status_item_emit_status_va (status,
 				_("Downloading article \"%s\" failed."),
-				article->subject);
+				article_get_subject(article));
 
 			/* if user's expecting to see something but we've failed,
 			   show them the error messages instead */
-			if (task->high_priority && status->errors!=NULL)
+			if (status->errors!=NULL)
 			{
 				GString * tmp = g_string_sized_new (1024);
 				GSList * l = status->errors;
@@ -319,16 +316,14 @@ task_body_run (Task * task)
 	***  Display the message?
 	**/
 
-	if (body->open_in_new_window) /* message window */
-	{
+	if (body->open_in_new_window)
+		 /* message window */
 		message_window_new (article, body->message_window_type);
-		article_set_read (article, TRUE);
-	}
-	else if (task->high_priority) /* use default text window */
-	{
+	else
+		/* use default text window */
 		text_update (article);
-		article_set_read (article, TRUE);
-	}
+
+	articles_set_read (&article, 1, TRUE);
 
 	debug_exit ("task_body_run");
 	return TASK_SUCCESS;
