@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team <pan@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,25 +29,20 @@
 #include <stdio.h>
 #include <time.h>
 
-#include <glib.h>
-#include <gtk/gtkbox.h>
-#include <gtk/gtkclist.h>
-#include <gtk/gtkscrolledwindow.h>
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
-#include <libgnomeui/gnome-dialog.h>
-#include <libgnomeui/gnome-stock.h>
-#include <libgnomeui/gnome-uidefs.h>
+#include <gnome.h>
 
-#include "acache.h"
-#include "article.h"
-#include "debug.h"
-#include "decode.h"
-#include "nntp.h"
-#include "prefs.h" /* for download_dir */
-#include "queue.h"
-#include "task-decode.h"
-#include "util.h" /* for pan_lock/unlock */
+#include <pan/base/acache.h>
+#include <pan/base/article.h>
+#include <pan/base/base-prefs.h>
+#include <pan/base/debug.h>
+#include <pan/base/decode.h>
+#include <pan/base/pan-glib-extensions.h>
+
+#include <pan/nntp.h>
+#include <pan/prefs.h> /* for auto_part_selection */
+#include <pan/queue.h>
+#include <pan/task-decode.h>
+#include <pan/util.h> /* for pan_lock/unlock */
 
 /*********************
 **********************  Defines / Enumerated types
@@ -96,10 +91,9 @@ task_decode_describe (const StatusItem* item)
 
 	/* sanity checks */
 	g_return_val_if_fail (item!=NULL, NULL);
+	g_return_val_if_fail (decode->readable_name!=NULL, NULL);
 
-        return decode->open
-		? g_strdup_printf (_("Opening \"%s\""), decode->readable_name)
-		: g_strdup_printf (_("Saving \"%s\""), decode->readable_name);
+        return g_strdup_printf (_("Saving \"%s\""), decode->readable_name);
 }
 
 /************
@@ -114,8 +108,6 @@ static void
 task_decode_destructor (PanObject* object)
 {
 	TaskDecode * task = TASK_DECODE(object);
-	GPtrArray * a;
-	GSList * l;
 
 	/* destruct the TaskDecode parts */
         debug1 (DEBUG_PAN_OBJECT, "task_decode destructor: %p", object);
@@ -124,11 +116,26 @@ task_decode_destructor (PanObject* object)
 	g_free (task->filename);
 
 	/* deflag */
-	a = g_ptr_array_new ();
-	for (l=task->articles; l!=NULL; l=l->next)
-		g_ptr_array_add (a, l->data);
-	articles_remove_flag ((Article**)a->pdata, a->len, STATE_DECODE_QUEUED);
-	g_ptr_array_free (a, TRUE);
+	if (1) {
+		GSList * l;
+		GPtrArray * a = g_ptr_array_new ();
+		for (l=task->articles; l!=NULL; l=l->next)
+			g_ptr_array_add (a, l->data);
+		articles_remove_flag ((Article**)a->pdata, a->len, STATE_DECODE_QUEUED);
+		g_ptr_array_free (a, TRUE);
+	}
+
+	/* checkin the article bodies */
+	if (1) {
+		GSList * l;
+		GPtrArray * a = g_ptr_array_new ();
+		for (l=task->articles; l!=NULL; l=l->next)
+			g_ptr_array_add (a, (gpointer)article_get_message_id(ARTICLE(l->data)));
+		acache_file_checkin ((const gchar**)a->pdata, a->len);
+		if (a->len>1 && task->runval==TASK_SUCCESS)
+			acache_expire_messages ((const gchar**)a->pdata, a->len);
+		g_ptr_array_free (a, TRUE);
+	}
 
 	g_slist_foreach (task->articles, (GFunc)pan_object_unref, NULL);
 	g_slist_free (task->articles);
@@ -143,8 +150,7 @@ task_decode_new (Group          * group,
                  const char     * readable_name, /* for description */
                  const char     * path,
                  const char     * filename,
-                 const GSList   * articles,
-                 gboolean         open)
+                 const GSList   * articles)
 {
 	GSList * l;
 	GPtrArray * a;
@@ -159,6 +165,16 @@ task_decode_new (Group          * group,
 	group_ref_articles (group, NULL);
 	g_slist_foreach ((GSList*)articles, (GFunc)pan_object_ref, NULL);
 
+	/* checkout the article bodies */
+	if (1) {
+		const GSList * l;
+		GPtrArray * a = g_ptr_array_new ();
+		for (l=articles; l!=NULL; l=l->next)
+			g_ptr_array_add (a, (gpointer)article_get_message_id(ARTICLE(l->data)));
+		acache_file_checkout ((const gchar**)a->pdata, a->len);
+		g_ptr_array_free (a, TRUE);
+	}
+
 	/* create the task */
        	task = g_new0 (TaskDecode, 1);
         debug1 (DEBUG_PAN_OBJECT, "task_decode_new: %p", task);
@@ -169,8 +185,8 @@ task_decode_new (Group          * group,
 			  group->server, FALSE, TRUE);
 
 	/* create the task-decode */
+	task->runval = TASK_FAIL;
 	task->group = group;
-	task->open = open;
 	task->readable_name = g_strdup(readable_name);
 	task->path = g_strdup (path);
 	task->filename = g_strdup (filename);
@@ -203,16 +219,24 @@ task_decode_new (Group          * group,
 static gint
 task_decode_run (Task * task)
 {
-	GSList * l;
-	Server * server = task->server;
-	Group * group = TASK_DECODE(task)->group;
-	GSList * articles = TASK_DECODE(task)->articles;
-	decode_data * dd = NULL;
-	Article * article;
 	gint result;
+	GSList * articles;
+	Server * server;
+	Group * group;
+	Article * article;
+	decode_data * dd = NULL;
+
+	/* sanity clause */
+	g_return_val_if_fail (task!=NULL, TASK_DECODE(task)->runval=TASK_FAIL_HOPELESS);
+	g_return_val_if_fail (TASK_DECODE(task)->articles!=NULL, TASK_DECODE(task)->runval=TASK_FAIL_HOPELESS);
+
+	server = task->server;
+	group = TASK_DECODE(task)->group;
+	articles = TASK_DECODE(task)->articles;
 
 	/* get the articles bodies */
-	if (1) {
+	if (1)
+	{
 		GSList * l;
 		gint index = 0;
 		GPtrArray * squarepeg = g_ptr_array_new ();
@@ -224,43 +248,34 @@ task_decode_run (Task * task)
 		                               &task->hint_abort,
 		                               squarepeg,
 		                               &index,
-		                               TRUE, TRUE);
+		                               FALSE, TRUE);
 		g_ptr_array_free (squarepeg, TRUE);
 	}
 	if (result != TASK_SUCCESS) {
 		status_item_emit_error_va (STATUS_ITEM(task),
 		                           _("Unable to decode `%s': the article(s) couldn't be downloaded."),
-					   ARTICLE(articles->data)->subject);
-		return result;
+					   article_get_subject(ARTICLE(articles->data)));
+		return TASK_DECODE(task)->runval = result;
 	}
 
 
 	/* update "read" attribute */
        	article = ARTICLE(articles->data);
-	article_set_read (article, TRUE); 
+	articles_set_read (&article, 1, TRUE); 
 
 	/* pass it off to article_decode */
 	dd = g_new0 (decode_data, 1);
 	dd->server = server;
 	dd->group = group;
 	dd->articles = articles;
-	dd->open = TASK_DECODE(task)->open;
 	dd->item = STATUS_ITEM(task);
 	dd->path = TASK_DECODE(task)->path;
 	dd->filename = TASK_DECODE(task)->filename;
 	decode_article (dd);
 	g_free (dd);
 
-	/* check in the articles */
-	if (1) {
-		GPtrArray * a = g_ptr_array_new ();
-		for (l=articles; l!=NULL; l=l->next)
-			g_ptr_array_add (a, (gpointer)ARTICLE(l->data)->message_id);
-		acache_file_checkin ((const gchar**)a->pdata, a->len);
-		g_ptr_array_free (a, TRUE);
-	}
 
-	return TASK_SUCCESS;
+	return TASK_DECODE(task)->runval = TASK_SUCCESS;
 }
 
 
@@ -397,14 +412,12 @@ choose_article_parts_manual (TaskDecode  * task,
 
 	pan_lock ();
 
-	ui->window = gnome_dialog_new (
-		_("Decode Multipart Message"),
-		GNOME_STOCK_BUTTON_OK,
-		GNOME_STOCK_BUTTON_CANCEL,
-		NULL);
-	gtk_signal_connect (
-		GTK_OBJECT(ui->window), "clicked",
-		GTK_SIGNAL_FUNC(select_articles_clicked_cb), ui);
+	ui->window = gnome_dialog_new (_("Pan: Decode Multipart Message"),
+	                               GNOME_STOCK_BUTTON_OK,
+	                               GNOME_STOCK_BUTTON_CANCEL,
+	                               NULL);
+	gtk_signal_connect (GTK_OBJECT(ui->window), "clicked",
+	                    GTK_SIGNAL_FUNC(select_articles_clicked_cb), ui);
 
 	gtk_window_set_policy (GTK_WINDOW (ui->window), TRUE, TRUE, TRUE);
 	gnome_dialog_set_default (GNOME_DIALOG(ui->window), 1);
@@ -438,7 +451,7 @@ choose_article_parts_manual (TaskDecode  * task,
 		sprintf (lines, "%d", article->linecount);
 		strftime (datebuf, sizeof(datebuf), "%Y/%m/%d %H:%M", &local_tm);
 
-	       	text[0] = (gchar*)article->subject;
+	       	text[0] = (gchar*) article_get_subject (article);
 		text[1] = lines;
 		text[2] = author;
 		text[3] = datebuf;
