@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team <pan@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,8 +20,8 @@
 #ifndef TASK_DECODE__H
 #define TASK_DECODE__H
 
-#include "task.h"
-#include "group.h"
+#include <pan/base/group.h>
+#include <pan/task.h>
 
 
 /**
@@ -36,9 +36,9 @@ typedef struct
 	Task         item;
 
 	/* private */
+	gint         runval;
 	Group      * group;
 	GSList     * articles;
-	gboolean     open;
 	gchar      * readable_name;
 	gchar      * path;
 	gchar      * filename;
@@ -54,8 +54,7 @@ PanObject* task_decode_new (Group        * group,
 			    const char   * readable_name_for_description,
 			    const char   * save_path_or_null,
 			    const char   * save_filename_or_null,
-			    const GSList * articles,
-			    gboolean       open);
+			    const GSList * articles);
 
 void task_decode_validate_and_queue_or_self_destruct (TaskDecode*, int queue_index);
 
