@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team <pan@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,16 +31,14 @@
 
 #include <glib.h>
 
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
-#include <libgnome/gnome-config.h>
+#include <pan/base/article.h>
+#include <pan/base/debug.h>
+#include <pan/base/log.h>
+#include <pan/base/pan-glib-extensions.h>
+#include <pan/base/pan-i18n.h>
 
-#include "article.h"
-#include "debug.h"
-#include "log.h"
-#include "nntp.h"
-#include "task-grouplist.h"
-#include "util.h"
+#include <pan/nntp.h>
+#include <pan/task-grouplist.h>
 
 /*********************
 **********************  Defines / Enumerated types
@@ -94,7 +92,6 @@ static void task_grouplist_destructor (PanObject*);
 
 PanObject*
 task_grouplist_new (Server *server,
-                    gboolean high_priority,
                     GrouplistDownloadType dl_type)
 {
 	TaskGroupList *item = g_new0 (TaskGroupList, 1);
@@ -105,7 +102,7 @@ task_grouplist_new (Server *server,
 	task_constructor (TASK(item),
 		task_grouplist_destructor,
 		task_grouplist_describe,
-		task_grouplist_run, server, high_priority, TRUE);
+		task_grouplist_run, server, FALSE, TRUE);
 
 	/* construct this class... */
 	item->server = server;
@@ -281,7 +278,7 @@ nntp_grouplist_download_all (TaskGroupList * item)
 			permission = get_next_token_str (march, ' ', &march);
 
 			/* create a new group object */
-			group = group_new (TASK(item)->server, name);
+			group = group_new (name);
 			group->permission = *permission;
 			group_set_dirty (group); /* make sure it gets saved */
 			g_ptr_array_add (groups, group);
@@ -352,7 +349,7 @@ nntp_grouplist_download_all (TaskGroupList * item)
 		response = pch;
 
 		/* get the description */
-		if ((pch = strstr (response, "\r\n"))) {
+		if ((pch = pan_strstr (response, "\r\n"))) {
 			description = g_strndup (response, pch-response);
 			g_strstrip (description);
 		}
@@ -384,9 +381,8 @@ nntp_grouplist_download_all (TaskGroupList * item)
 			if (exact_match)
 			{
 				Group * g = GROUP(g_ptr_array_index(groups,i));
-				group_set_string_field (g,
-				                        &g->description,
-				                        description);
+				replace_gstr (&g->description, description);
+				description = NULL;
 			}
 		}
 
@@ -461,7 +457,7 @@ nntp_grouplist_download_new (TaskGroupList * item)
 
 		/* create a Group */
 		name = get_next_token_str (response, ' ', NULL);
-		group = group_new (TASK(item)->server, name);
+		group = group_new (name);
 		group_set_dirty (group); /* make sure it gets saved */
 		g_free (name);
 		group->flags |= GROUP_NEW;
