@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team <pan@rebelbas.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,16 +30,16 @@
 #include <string.h>
 
 #include <glib.h>
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
 
-#include "article.h"
-#include "debug.h"
-#include "log.h"
-#include "nntp.h"
-#include "rule-manager.h"
-#include "task-headers.h"
-#include "util.h"
+#include <pan/base/article.h>
+#include <pan/base/debug.h>
+#include <pan/base/log.h>
+#include <pan/base/pan-glib-extensions.h>
+#include <pan/base/pan-i18n.h>
+
+#include <pan/nntp.h>
+#include <pan/task-headers.h>
+#include <pan/rules/rule-manager.h>
 
 /*********************
 **********************  Defines / Enumerated types
@@ -278,8 +278,8 @@ task_headers_run (Task* task)
 			/* add the articles to the group */
 			group_mark_new_article_number (task_h->group, task_h->group->article_high);
 			group_add_articles_remove_unused (task_h->group, articles, status);
+			group_expire_articles_not_in_range (task_h->group, low_in_group, high_in_group);
 			group_set_article_range (task_h->group, low_in_group, high_in_group);
-			group_expire_old_articles (task_h->group);
 			group_thread_if_needed (task_h->group, status);
 
 			/* apply the rules to the new headers */
