@@ -1,6 +1,8 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
+ * Author: Charles Kerr <charles@rebelbase.com>
+ *
+ * Copyright (C) 2000, 2001  Pan Development Team <pan@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,34 +25,21 @@
 
 #include <config.h>
 
-#include <glib.h>
-#include <gtk/gtkclist.h>
-#include <gtk/gtkmain.h>
-#include <gtk/gtkscrolledwindow.h>
-#include <gtk/gtkvseparator.h>
-#include <gtk/gtkwindow.h>
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
-#include <libgnomeui/gnome-pixmap.h>
-#include <libgnomeui/gnome-stock.h>
-#include <libgnomeui/gnome-uidefs.h>
+#include <gnome.h>
 
-#include "debug.h"
-#include "gui.h"
-#include "prefs.h"
-#include "queue.h"
-#include "status-item-view.h"
-#include "task-manager.h"
-#include "util.h"
+#include <pan/base/debug.h>
+#include <pan/base/pan-glib-extensions.h>
 
-#include "xpm/arrow_up.xpm"
-#include "xpm/arrow_top.xpm"
-#include "xpm/arrow_down.xpm"
-#include "xpm/arrow_bottom.xpm"
-#include "xpm/delete_row.xpm"
-#include "xpm/delete_sheet.xpm"
-#include "xpm/requeue.xpm"
-#include "xpm/server.xpm"
+#include <pan/gui.h>
+#include <pan/prefs.h>
+#include <pan/queue.h>
+#include <pan/status-item-view.h>
+#include <pan/task-manager.h>
+#include <pan/util.h>
+
+#include <pan/xpm/delete_row.xpm>
+#include <pan/xpm/delete_sheet.xpm>
+#include <pan/xpm/server.xpm>
 
 /*********************
 **********************  Defines / Enumerated types
@@ -70,7 +59,11 @@ typedef struct
 	GtkWidget    * top;
 	GtkWidget    * list;
 
+	GtkWidget    * pause_pixmap;
+	GtkWidget    * ready_pixmap;
+
 	guint          timeout_id;
+	guint          moved_signal_id;
 
 	gboolean       titlebar_needs_refresh;
 	gboolean       dampen_move_feedback_loop;
@@ -144,19 +137,142 @@ get_selected_tasks (ManagerUI * ui)
 	return a;
 }
 
+/**
+*** Helper functions for user interactions
+**/
+
+static void
+move_selected_tasks_to_top (ManagerUI * ui)
+{
+	GtkCList * clist;
+	GList * l;
+	gint new_row;
+	debug_enter ("move_selected_tasks_to_top");
+
+	/* sanity clause */
+	g_return_if_fail (ui!=NULL);
+
+	clist = GTK_CLIST(ui->list);
+	new_row = -1;
+	for (l=clist->selection; l!=NULL; l=l->next)
+	{
+		const int old_row = GPOINTER_TO_INT (l->data);
+		Task * const task = gtk_clist_get_row_data (clist, old_row);
+		queue_task_move (task, ++new_row);
+	}
+
+	debug_exit ("move_selected_tasks_to_top");
+}
+
+static void
+move_selected_tasks_to_bottom (ManagerUI * ui)
+{
+	GtkCList * clist;
+	GList * l;
+	gint new_row;
+	debug_enter ("move_selected_tasks_to_bottom");
+
+	/* sanity clause */
+	g_return_if_fail (ui!=NULL);
+
+	clist = GTK_CLIST(ui->list);
+	new_row = 0;
+	for (l=clist->selection; l!=NULL; l=l->next)
+	{
+		const int old_row = GPOINTER_TO_INT (l->data);
+		Task * const task = gtk_clist_get_row_data (clist, old_row);
+		queue_task_move (task, clist->rows - ++new_row);
+	}
+
+	debug_exit ("move_selected_tasks_to_bottom");
+}
+
+static void
+cancel_selected_tasks (ManagerUI * ui)
+{
+	GPtrArray * a;
+        debug_enter ("cancel_selected_tasks");
+
+	/* sanity clause */
+	g_return_if_fail (ui!=NULL);
+
+	a = get_selected_tasks (ui);
+	pan_g_ptr_array_foreach (a, (GFunc)queue_task_remove, NULL);
+	g_ptr_array_free (a, TRUE);
+
+        debug_exit ("cancel_selected_tasks");
+}
+
+static void
+select_task (ManagerUI * ui, gboolean first)
+{
+	GtkCList * clist;
+	gint       row;
+	debug_enter ("select_task");
+
+	/* sanity clause */
+	g_return_if_fail (ui!=NULL);
+
+	clist = GTK_CLIST(ui->list);
+	row = first ? 0 : clist->rows-1;
+	
+	gtk_clist_freeze(clist);
+	gtk_clist_unselect_all (clist);
+	gtk_clist_select_row (clist, row, 0);
+	clist->focus_row = row;
+	gtk_clist_thaw(clist);
+
+	debug_exit("select_task");
+}
+
+
 /**
 ***  User Interactions
 **/
 
+static void
+decorate_pause_button (ManagerUI * ui, GtkToggleButton * tb, gboolean paused)
+{
+	GtkWidget * w;
+
+	/* update pixmap */
+	w = GTK_BIN(tb)->child;
+	if (w != NULL)
+		gtk_container_remove (GTK_CONTAINER(tb), w);
+	gtk_container_add (GTK_CONTAINER(tb), paused ? ui->pause_pixmap : ui->ready_pixmap);
+	gtk_widget_show_all (GTK_WIDGET(tb));
+
+	/* update tooltip */
+	gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), GTK_WIDGET(tb), paused
+		? _("The Queue is Paused")
+		: _("The Queue is not Paused"), "");
+}
+static void
+pause_toggled_cb (GtkToggleButton  * tb,
+                  gpointer           user_data)
+{
+	ManagerUI * ui = (ManagerUI*) user_data;
+	gboolean paused = gtk_toggle_button_get_active (tb);
+
+	if (paused != queue_is_paused())
+		queue_set_paused (paused);
+
+	decorate_pause_button (ui, tb, paused);
+}
+
 static void
 up_clicked_cb (GtkButton   * button,
                gpointer      user_data)
 {
-        ManagerUI *ui = (ManagerUI*) user_data;
-	GtkCList *clist = GTK_CLIST(ui->list);
-	GList *l;
-	int last_index = -1;
+        ManagerUI * ui;
+	GtkCList * clist;
+	GList * l;
+	gint last_index;
+	debug_enter ("up_clicked_cb");
 
+        ui = (ManagerUI*) user_data;
+	clist = GTK_CLIST(ui->list);
+	last_index = -1;
 	for (l=clist->selection; l!=NULL; l=l->next)
 	{
 		const int index = GPOINTER_TO_INT (l->data);
@@ -165,74 +281,73 @@ up_clicked_cb (GtkButton   * button,
 		queue_task_move (task, new_index);
 		last_index = new_index;
 	}
+
+	debug_exit ("up_clicked_cb");
 }
 
 static void
 top_clicked_cb (GtkButton    * button,
                 gpointer       user_data)
 {
-        ManagerUI *ui = (ManagerUI*) user_data;
-	GtkCList *clist = GTK_CLIST(ui->list);
-	GList *l;
-	int new_row = -1;
+	debug_enter ("top_clicked_cb");
 
-	for (l=clist->selection; l!=NULL; l=l->next)
-	{
-		const int old_row = GPOINTER_TO_INT (l->data);
-		Task * const task = gtk_clist_get_row_data (clist, old_row);
-		queue_task_move (task, ++new_row);
-	}
+	move_selected_tasks_to_top ((ManagerUI *) user_data);
+
+	debug_exit ("top_clicked_cb");
 }
 
 static void
 down_clicked_cb (GtkButton   * button,
                  gpointer      user_data)
 {
-	ManagerUI *ui = (ManagerUI*) user_data;
-	GtkCList *clist = GTK_CLIST(ui->list);
-	GList *l;
+	ManagerUI * ui;
+	GtkCList * clist;
+	GList * l;
+	debug_enter ("down_clicked_cb");
+
+	ui = (ManagerUI*) user_data;
+	clist = GTK_CLIST(ui->list);
 	for (l=g_list_last(clist->selection); l!=NULL; l=l->prev)
 	{
 		const int index = GPOINTER_TO_INT (l->data);
 		Task * const task = gtk_clist_get_row_data (clist, index);
 		queue_task_move (task, index+1);
 	}
+
+	debug_exit ("down_clicked_cb");
 }
 
 static void
 bottom_clicked_cb (GtkButton   * button,
                    gpointer      user_data)
 {
-        ManagerUI *ui = (ManagerUI*) user_data;
-	GtkCList *clist = GTK_CLIST(ui->list);
-	GList *l;
-	int new_row = 0;
+	debug_enter ("bottom_clicked_cb");
 
-	for (l=clist->selection; l!=NULL; l=l->next)
-	{
-		const int old_row = GPOINTER_TO_INT (l->data);
-		Task * const task = gtk_clist_get_row_data (clist, old_row);
-		queue_task_move (task, clist->rows - ++new_row);
-	}
+	move_selected_tasks_to_bottom ((ManagerUI *) user_data);
+
+	debug_exit ("bottom_clicked_cb");
 }
 
 static void
 cancel_clicked_cb (GtkButton   * button,
                    gpointer      user_data)
 {
-	ManagerUI *ui = (ManagerUI*) user_data;
-	GPtrArray * a = get_selected_tasks (ui);
-	pan_g_ptr_array_foreach (a, (GFunc)queue_task_remove, NULL);
-	g_ptr_array_free (a, TRUE);
+	debug_enter ("cancel_clicked");
+
+	cancel_selected_tasks ((ManagerUI *) user_data);
+
+	debug_exit ("cancel_clicked");
 }
 
 static void
 clear_queue_clicked_cb (GtkButton   * b,
                         gpointer      user_data)
 {
-	GSList * queue = g_slist_copy ((GSList*)queue_get_tasks());
-	GSList * l = NULL;
+	GSList * l;
+	GSList * queue;
+	debug_enter ("clear_queue_clicked_cb");
 
+	queue = g_slist_copy ((GSList*)queue_get_tasks());
 	for (l=queue; l!=NULL; l=l->next) {
 		Task * task = TASK(l->data);
 		if (queue_get_task_status(task)==QUEUE_TASK_STATUS_QUEUED)
@@ -240,21 +355,29 @@ clear_queue_clicked_cb (GtkButton   * b,
 	}
 
 	g_slist_free (queue);
+
+	debug_exit ("clear_queue_clicked_cb");
 }
 
 static void
 requeue_cb (GtkButton   * button,
             gpointer      user_data)
 {
-	ManagerUI * ui = (ManagerUI*) user_data;
-	GPtrArray * a = get_selected_tasks (ui);
 	guint i;
+	ManagerUI * ui;
+	GPtrArray * a;
+	debug_enter ("requeue_cb");
+
+	ui = (ManagerUI*) user_data;
+	a = get_selected_tasks (ui);
 	for (i=0; i!=a->len; ++i) {
 		Task * task = (Task*) g_ptr_array_index (a, i);
 		if (queue_get_task_status(task) == QUEUE_TASK_STATUS_FAILED)
 			queue_task_requeue_failed (task, 0);
 	}
 	g_ptr_array_free (a, TRUE);
+
+	debug_exit ("requeue_cb");
 }
 
 
@@ -309,7 +432,7 @@ get_xfer_str (const Task    * task,
 		const int m = (secs_left % 3600) / 60;
 		const int s = secs_left % 60;
 		const double KBps = pan_socket_get_xfer_rate_KBps (task->sock);
-		sprintf (buf, _("%.2f KBps, %d:%02d:%02d remaining"), KBps, h, m, s);
+		sprintf (buf, _("%.2f KB/s, %d:%02d:%02d remaining"), KBps, h, m, s);
 	}
 }
 
@@ -401,10 +524,16 @@ queue_row_move_cb (GtkCList   * clist,
                    int          new_index,  
                    gpointer     user_data)
 {
-	ManagerUI *ui = (ManagerUI*) user_data;
-	Task *task = TASK(gtk_clist_get_row_data (clist, old_index));
+	ManagerUI * ui;
+	Task * task;
+	debug_enter ("queue_row_move_cb");
+
+	ui = (ManagerUI*) user_data;
+	task = TASK(gtk_clist_get_row_data (clist, old_index));
 	if (!ui->dampen_move_feedback_loop)
 		queue_task_move (task, new_index);
+
+	debug_exit ("queue_row_move_cb");
 }
 
 
@@ -454,12 +583,59 @@ task_moved_cb (gpointer call_obj,
 	int new_row = ((int*)call_arg)[1];
 
 	pan_lock ();
+	gtk_signal_disconnect (GTK_OBJECT(ui->list), ui->moved_signal_id);
 	gtk_clist_row_move (GTK_CLIST(ui->list), old_row, new_row);
+	ui->moved_signal_id = gtk_signal_connect (GTK_OBJECT(ui->list), "row_move", queue_row_move_cb, ui);
 	pan_unlock ();
 
 	return 0;
 }
 
+static gboolean
+gui_key_press_cb (GtkWidget      * widget,
+                  GdkEventKey    * event,
+                  gpointer         data)
+{
+	ManagerUI * ui;
+	GtkCList  * clist;
+	debug_enter ("gui_key_press_cb");
+
+	ui    = (ManagerUI*) data;
+	clist = GTK_CLIST (ui->list);
+
+        switch (event->keyval)
+        {
+                case GDK_Delete:
+			cancel_selected_tasks (ui);
+			break;
+                case GDK_Home:
+                        if (event->state & GDK_SHIFT_MASK)
+			{
+				move_selected_tasks_to_top (ui);
+			}
+			else
+			{
+				select_task (ui, TRUE);
+			}
+			break;
+                case GDK_End:
+                        if (event->state & GDK_SHIFT_MASK)
+			{
+				move_selected_tasks_to_bottom (ui);
+			}
+			else
+			{
+				select_task (ui, FALSE);
+			}
+			break;
+                default:
+			break;
+        }
+
+	debug_exit ("gui_key_press_cb");
+        return TRUE;
+}
+
 static void
 update_titlebar_lock_unconditional (ManagerUI * ui)
 {
@@ -572,33 +748,57 @@ ui_destroy_cb (GtkObject * o, gpointer user_data)
 **/
 
 static ManagerUI*
-task_manager_build_ui (void)
+task_manager_build_ui (GtkWidget * window)
 {
+	gboolean b;
 	ManagerUI * ui = g_new0 (ManagerUI, 1);
 	GtkWidget * w;
+	GtkWidget * p;
 	GtkWidget * hbox;
 	GtkWidget * vbox;
 	char* list_titles [5];
 
+	ui->window = window;
 	vbox = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
 
+	/* create the ready/pause pixmaps */
+	ui->ready_pixmap = gnome_stock_pixmap_widget_new (ui->window, GNOME_STOCK_PIXMAP_TIMER);
+	ui->pause_pixmap = gnome_stock_pixmap_widget_new (ui->window, GNOME_STOCK_PIXMAP_TIMER_STOP);
+	gtk_object_ref (GTK_OBJECT(ui->ready_pixmap));
+	gtk_object_ref (GTK_OBJECT(ui->pause_pixmap));
+
 	/* button bar */
 	hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
 	gtk_container_set_border_width (GTK_CONTAINER(hbox), GNOME_PAD_SMALL);
 
+	/* pause button */
+	b = queue_is_paused ();
+	w = gtk_toggle_button_new ();
+	gtk_button_set_relief (GTK_BUTTON(w), GTK_RELIEF_NONE);
+	gtk_signal_connect (GTK_OBJECT(w), "toggled", pause_toggled_cb, ui);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), b);
+	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, GNOME_PAD);
+	decorate_pause_button (ui, GTK_TOGGLE_BUTTON(w), b);
+
+	/* separator */
+	w = gtk_vseparator_new ();
+	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, GNOME_PAD);
+
 	/* up button */
 	w = gtk_button_new ();
+	p = gnome_stock_pixmap_widget_new (window, GNOME_STOCK_PIXMAP_UP);
 	gtk_button_set_relief (GTK_BUTTON(w), GTK_RELIEF_NONE);
 	gtk_signal_connect (GTK_OBJECT(w), "clicked", up_clicked_cb, ui);
-	gtk_container_add (GTK_CONTAINER(w), gnome_pixmap_new_from_xpm_d (arrow_up_xpm));
+	gtk_container_add (GTK_CONTAINER(w), p);
 	gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, _("Move Selected Task(s) Up"), "");
 	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, 0);
 
 	/* top button */
 	w = gtk_button_new ();
+	p = gnome_stock_pixmap_widget_new (window, GNOME_STOCK_PIXMAP_TOP);
 	gtk_button_set_relief (GTK_BUTTON(w), GTK_RELIEF_NONE);
 	gtk_signal_connect (GTK_OBJECT(w), "clicked", top_clicked_cb, ui);
-	gtk_container_add (GTK_CONTAINER(w), gnome_pixmap_new_from_xpm_d (arrow_top_xpm));
+	gtk_container_add (GTK_CONTAINER(w), p);
 	gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, _("Move Selected Task(s) To Top"), "");
 	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, 0);
 
@@ -608,16 +808,18 @@ task_manager_build_ui (void)
 
 	/* down button */
 	w = gtk_button_new ();
+	p = gnome_stock_pixmap_widget_new (window, GNOME_STOCK_PIXMAP_DOWN);
 	gtk_signal_connect (GTK_OBJECT(w), "clicked", down_clicked_cb, ui);
-	gtk_container_add (GTK_CONTAINER(w), gnome_pixmap_new_from_xpm_d (arrow_down_xpm));
+	gtk_container_add (GTK_CONTAINER(w), p);
 	gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, _("Move Selected Task(s) Down"), "");
 	gtk_button_set_relief (GTK_BUTTON(w), GTK_RELIEF_NONE);
 	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, 0);
 
 	/* bottom button */
 	w = gtk_button_new ();
+	p = gnome_stock_pixmap_widget_new (window, GNOME_STOCK_PIXMAP_BOTTOM);
 	gtk_signal_connect (GTK_OBJECT(w), "clicked", bottom_clicked_cb, ui);
-	gtk_container_add (GTK_CONTAINER(w), gnome_pixmap_new_from_xpm_d (arrow_bottom_xpm));
+	gtk_container_add (GTK_CONTAINER(w), p);
 	gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, _("Move Selected Task(s) To Bottom"), "");
 	gtk_button_set_relief (GTK_BUTTON(w), GTK_RELIEF_NONE);
 	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, 0);
@@ -628,8 +830,9 @@ task_manager_build_ui (void)
 
 	/* requeue button */
 	w = gtk_button_new ();
+	p = gnome_stock_pixmap_widget_new (window, GNOME_STOCK_PIXMAP_REDO);
 	gtk_signal_connect (GTK_OBJECT(w), "clicked", requeue_cb, ui);
-	gtk_container_add (GTK_CONTAINER(w), gnome_pixmap_new_from_xpm_d (requeue_xpm));
+	gtk_container_add (GTK_CONTAINER(w), p);
 	gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, _("Requeue selected failed Task(s)"), "");
 	gtk_button_set_relief (GTK_BUTTON(w), GTK_RELIEF_NONE);
 	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, 0);
@@ -677,7 +880,7 @@ task_manager_build_ui (void)
 	gui_restore_column_widths (ui->list, "task_manager_2");
 
 	gtk_clist_set_selection_mode (GTK_CLIST(w), GTK_SELECTION_EXTENDED);
-	gtk_signal_connect (GTK_OBJECT(w), "row_move", queue_row_move_cb, ui);
+	ui->moved_signal_id = gtk_signal_connect (GTK_OBJECT(w), "row_move", queue_row_move_cb, ui);
 	w = gtk_scrolled_window_new (NULL, NULL);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(w), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	gtk_container_add (GTK_CONTAINER (w), ui->list);
@@ -733,9 +936,8 @@ task_manager_spawn (void)
 		return;
 	}
 
-	manager_ui = task_manager_build_ui ();
+	manager_ui = task_manager_build_ui (gtk_window_new(GTK_WINDOW_TOPLEVEL));
 	ui_populate (manager_ui);
-	manager_ui->window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 
 	pan_callback_add (queue_task_added, update_titlebar_cb, manager_ui);
 	pan_callback_add (queue_task_removed, update_titlebar_cb, manager_ui);
@@ -744,6 +946,8 @@ task_manager_spawn (void)
 	                    GTK_SIGNAL_FUNC(window_delete_event_cb), manager_ui);
 	gtk_signal_connect (GTK_OBJECT(manager_ui->window), "destroy",
 	                    GTK_SIGNAL_FUNC(remove_update_titlebar_callbacks_cb), manager_ui);
+	gtk_signal_connect (GTK_OBJECT (manager_ui->window), "key_press_event",
+	                    GTK_SIGNAL_FUNC(gui_key_press_cb), manager_ui);
 	gtk_window_set_title (GTK_WINDOW(manager_ui->window), _("Pan - Task Manager"));
 	gtk_container_add (GTK_CONTAINER(manager_ui->window), manager_ui->top);
 
