@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team <pan@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,9 +28,10 @@
 
 #include <glib.h>
 
-#include "debug.h"
-#include "task.h"
-#include "util.h"
+#include <pan/base/debug.h>
+#include <pan/base/pan-glib-extensions.h>
+
+#include <pan/task.h>
 
 /*********************
 **********************  Defines / Enumerated types
@@ -151,9 +152,9 @@ task_run (Task * task)
 *****/
 
 void
-task_hint_abort (Task* task, gboolean abort)
+task_hint_abort (Task * task)
 {
 	g_return_if_fail (task!=NULL);
 
-	task->hint_abort = abort;
+	task->hint_abort = TRUE;
 }
