@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team <pan@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,10 +25,11 @@
 
 #include <glib.h>
 
-#include "pan-callback.h"
-#include "server.h"
-#include "sockets.h"
-#include "status-item.h"
+#include <pan/base/pan-callback.h>
+#include <pan/base/server.h>
+#include <pan/base/status-item.h>
+
+#include <pan/sockets.h>
 
 /**
 ***  TASK CLASS DEFINITION
@@ -87,7 +88,6 @@ void task_constructor     (Task                      * task,
 
 int          task_run                  (Task           * task);
 
-void         task_hint_abort           (Task           * task,
-                                        gboolean         do_abort);
+void         task_hint_abort           (Task           * task);
 
 #endif
