@@ -23,7 +23,6 @@
 #include <ctype.h>
 #include <string.h>
 
-#include <glib.h>
 #include <gnome.h>
 
 #ifdef USE_GTKHTML
@@ -38,23 +37,25 @@
 #include <gal/widgets/e-font.h>
 #endif
 
-#include "../gmime/gmime.h"
+#include <gmime/gmime.h>
 
-#include "articlelist.h"
-#include "debug.h"
-#include "gui.h"
-#include "gui-headers.h"
-#include "globals.h"
-#include "grouplist.h"
-#include "message-window.h"
-#include "pan-object.h"
-#include "prefs.h"
-#include "queue.h"
-#include "task-body.h"
-#include "text.h"
-#include "util.h"
-#include "util-wrap.h"
-#include "util-mime.h"
+#include <pan/base/debug.h>
+#include <pan/base/pan-glib-extensions.h>
+#include <pan/base/pan-object.h>
+#include <pan/base/util-file.h>
+#include <pan/base/util-mime.h>
+#include <pan/base/util-wrap.h>
+
+#include <pan/articlelist.h>
+#include <pan/gui.h>
+#include <pan/gui-headers.h>
+#include <pan/globals.h>
+#include <pan/message-window.h>
+#include <pan/prefs.h>
+#include <pan/queue.h>
+#include <pan/task-body.h>
+#include <pan/text.h>
+#include <pan/util.h>
 
 static GdkColor headers_col = {0, 0xbbbb, 0, 0};
 #ifndef USE_GTKHTML
@@ -64,8 +65,6 @@ static GtkWidget * header_table = NULL;
 static void ensure_header_col_inited (void);
 static void realize_text_cb (GtkWidget *text, gpointer data);
 static void text_button_press_cb (GtkWidget *widget, GdkEventButton *bevent);
-static void text_key_press_cb (GtkWidget *widget, GdkEventKey *event);
-static void text_page_down_or_next_article (GtkWidget *);
 static void show_all_headers_cb (GtkCheckMenuItem* item, gpointer data);
 static void mute_quoted_text_cb (GtkCheckMenuItem* item, gpointer data);
 
@@ -134,6 +133,14 @@ static GnomeUIInfo filter_menu[] =
 	GNOMEUIINFO_END
 };
 
+static void
+text_read_body_in_new_window (void)
+{
+	Article * article = get_current_article ();
+	if (article != NULL)
+		message_read_window (article);
+}
+
 
 static GnomeUIInfo text_menu_popup[] =
 {
@@ -141,7 +148,7 @@ static GnomeUIInfo text_menu_popup[] =
 		GNOME_APP_UI_ITEM,
 		N_("Open in New Window"),
 		N_("Open this message for reading."),
-		message_read_window
+		text_read_body_in_new_window
 	},
 	GNOMEUIINFO_SEPARATOR, /* 1 */
         { /* 2 */
@@ -210,10 +217,8 @@ static gchar welcome[] =
 "PURPOSE.  See the GNU General Public License for more "
 "details."
 "<p>"
-"You should have received a copy of the GNU General Public "
-"License along with this program; if not, write to the "
-"Free Software Foundation, Inc., 59 Temple Place, "
-"Suite 330, Boston, MA 02111-1307, USA"
+"The GNU Public License can be found from the menu above "
+"in Help|About|License."
 "</font></body></html>"
 #else
 "Pan " VERSION "\n"
@@ -238,10 +243,8 @@ static gchar welcome[] =
 "PURPOSE.  See the GNU General Public License for more \n"
 "details. \n"
 "\n"
-"You should have received a copy of the GNU General Public \n"
-"License along with this program; if not, write to the \n"
-"Free Software Foundation, Inc., 59 Temple Place, \n"
-"Suite 330, Boston, MA 02111-1307, USA"
+"The GNU Public License can be found from the menu above \n"
+"in Help|About|License. \n"
 #endif
 };
 
@@ -284,57 +287,59 @@ text_select_all (void)
 ***  HTML
 **/
 
-#ifdef USE_GTKHTML
-
 static void
-text_page_down_or_next_article (GtkWidget * w)
+sylpheed_textview_smooth_scroll_do (GtkAdjustment  * vadj,
+                                    gfloat           old_value,
+                                    gfloat           last_value,
+                                    gint             step)
 {
+	gint i;
+	gint change_value;
+	gboolean up;
+
+	if (old_value < last_value) {
+		change_value = last_value - old_value;
+		up = FALSE;
+	} else {
+		change_value = old_value - last_value;
+		up = TRUE;
+	}
+
+	gdk_key_repeat_disable ();
+
+	for (i=step; i<=change_value; i+=step)
+		gtk_adjustment_set_value (vadj, old_value+(up?-i:i));
+	gtk_adjustment_set_value (vadj, last_value);
+
+	gdk_key_repeat_restore ();
+}
+
+void
+text_read_more (void)
+{
+#ifdef USE_GTKHTML
+	GtkAdjustment * v = GTK_LAYOUT(Pan.text)->vadjustment;
+#else
+	GtkAdjustment * v = GTK_TEXT(Pan.text)->vadj;
+#endif
+
+	/* try to go down a page */
+	const gfloat val = CLAMP (v->value + v->page_size,
+	                          v->lower,
+	                          MAX(v->upper,v->page_size)-MIN(v->upper,v->page_size));
+
+	/* if successful, then update the viewpane; otherwise, go to the next article */
+	if (val == v->value)
+		articlelist_view_next ();
+	else if (text_window_smooth_scrolling)
+		sylpheed_textview_smooth_scroll_do (v, v->value, val,
+		                                    text_window_smooth_scrolling_speed);
+	else
+		gtk_adjustment_set_value (v, val);
 }
 
 
-static gchar*          
-make_safe_for_html (const gchar * str)
-{
-	gint size_needed;
-	gchar * retval;
-	gchar * out;
-	const gchar * in;
-	debug_enter ("make_safe_for_html");
-
-	/* sanity clause */
-	if (!is_nonempty_string (str))
-		return g_strdup ("");
-
-	/* how big must the buffer be? */
-	size_needed = 1;
-	for (in=str; *in; ++in) {
-		gint inc;
-		switch (*in) {
-			case '&': inc = 5; break;
-			case '>': case '<': inc = 4; break;
-			case '"': inc = 6; break;
-			default: inc = 1; break;
-		}
-		size_needed += inc;
-	}
-
-	/* build the output string */
-	retval = out = g_malloc (size_needed);
-	for (in=str; *in; ++in) {
-		switch (*in) {
-			case '&': memcpy (out, "&amp;", 5); out += 5; break;
-			case '>': memcpy (out, "&gt;", 4); out += 4; break;
-			case '<': memcpy (out, "&lt;", 4); out += 4; break;
-			case '"': memcpy (out, "&quot;", 6); out += 6; break;
-			default: *out++ = *in; break;
-		}
-	}
-	*out = '\0';
-
-	debug_exit ("make_safe_for_html");
-	return retval;
-}
-
+#ifdef USE_GTKHTML
 
 static void
 add_header (GString * buf, const gchar * header_name, const gchar * val)
@@ -342,7 +347,7 @@ add_header (GString * buf, const gchar * header_name, const gchar * val)
 	gchar * tmp_val;
 	debug_enter ("add_header");
 
-       	tmp_val = make_safe_for_html (val);
+       	tmp_val = pan_str_escape (val);
 	if (is_nonempty_string (tmp_val))
 		g_string_sprintfa (buf, "<tr><td><b>%s</b> %s</td></tr>", header_name, tmp_val);
 
@@ -366,8 +371,8 @@ add_inline_images_mpfunc (GMimePart * part, gpointer user_data)
 	const gchar * content_id;
 	const GMimeContentType * type;
 
-	if (pan_strcmp ("inline", g_mime_part_get_content_disposition (part)))
-		return;
+	/*if (pan_strcmp ("inline", g_mime_part_get_content_disposition (part)))
+		return;*/ /* let all images get shown */
 
 	type = g_mime_part_get_content_type (part);
 	if (!g_mime_content_type_is_type (type, "image", "*"))
@@ -376,7 +381,7 @@ add_inline_images_mpfunc (GMimePart * part, gpointer user_data)
 	content_id = g_mime_part_get_content_id (part);
 	if (!is_nonempty_string(content_id)) /* uh oh, no cid, let's make one */
 	{
-		GString * id = g_string_new (data->a->message_id);
+		GString * id = g_string_new (article_get_message_id(data->a));
 		g_string_erase (id, id->len-1, 1); /* '>' */
 		g_string_sprintfa (id, "___%d>", data->i++);
 		g_mime_part_set_content_id (part, id->str);
@@ -387,6 +392,45 @@ add_inline_images_mpfunc (GMimePart * part, gpointer user_data)
 	g_string_sprintfa (str, "<img alt=\"%s\" src=\"cid:%s\"><p>\n", content_id, content_id);
 }
 
+static gchar*
+colorize_html (const gchar * body)
+{
+	gint line_len;
+	const gchar * line_start;
+	GString * buf;
+	gchar * retval;
+	gchar quoted_color[64];
+	gchar normal_color[64];
+
+	/* sanity clause */
+	g_return_val_if_fail (body!=NULL, NULL);
+
+	/* init the color strings */
+	sprintf (quoted_color, "<font color=\"#%02x%02x%02x\">",
+	                       text_quoted_color.red   >>8,
+	                       text_quoted_color.green >>8,
+	                       text_quoted_color.blue  >>8);
+	sprintf (normal_color, "<font color=\"#%02x%02x%02x\">",
+	                       text_fg_color.red   >>8,
+	                       text_fg_color.green >>8,
+	                       text_fg_color.blue  >>8);
+
+	/* colorize each line */
+	line_len = 0;
+	line_start = NULL;
+	buf = g_string_sized_new (strlen(body) * 2);
+	while (get_next_token_range (body, '\n', &body, &line_start, &line_len)) {
+		const gboolean is_reply = !strncmp (line_start, "&gt;", 4);
+		g_string_append (buf, is_reply ? quoted_color : normal_color);
+		pan_g_string_append_len (buf, line_start, line_len);
+		g_string_append (buf, "</font>\n");
+	}
+
+	retval = buf->str;
+	g_string_free (buf, FALSE);
+	return retval;
+}
+
 static void
 text_get_article_body (const Article * a, GString * buf)
 {
@@ -407,7 +451,7 @@ text_get_article_body (const Article * a, GString * buf)
 		const gchar * content_id = g_mime_part_get_content_id (mp);
 		if (!is_nonempty_string(content_id))
 		{
-			GString * id = g_string_new (a->message_id);
+			GString * id = g_string_new (article_get_message_id(a));
 			g_string_erase (id, id->len-1, 1); /* '>' */
 			g_string_sprintfa (id, "___%d>", 0);
 			g_mime_part_set_content_id (mp, id->str);
@@ -450,37 +494,36 @@ text_get_article_body (const Article * a, GString * buf)
 				replace_gstr (&body, mute_quoted_text_in_body(body));
 			if (text_get_wrap())
 				replace_gstr (&body, fill_body(body,wrap_column));
-			replace_gstr (&body, make_safe_for_html (body));
+			replace_gstr (&body, pan_str_escape (body));
+			replace_gstr (&body, colorize_html (body));
 			replace_gstr (&body, linkify_text(body));
 		}
 
-		/* attachments */
-		if (1)
-		{
-			AttachmentWorkStruct tmp;
-			tmp.i = 0;
-			tmp.a = a;
-			tmp.str = g_string_new (NULL);
-			g_mime_part_foreach (mp, add_inline_images_mpfunc, &tmp);
-			if (tmp.str->len)
-				replace_gstr (&body, g_strdup_printf("%s%s", tmp.str->str, (body?body:"")));
-			g_string_free (tmp.str, TRUE);
-		}
-
 		/* the headers */
 		if (text_get_show_all_headers())
 		{
 			gchar * headers = g_mime_message_get_headers (mm);
-			replace_gstr (&headers, make_safe_for_html (headers));
+			replace_gstr (&headers, pan_str_escape (headers));
 			replace_gstr (&body, g_strdup_printf ("%s\n\n%s", headers, (body?body:"")));
 			g_free (headers);
 		}
-			
 
 		/* add the body */
 		g_string_append (buf, "<pre>");
 		g_string_append (buf, body?body:"");
 		g_string_append (buf, "</pre>");
+
+		/* attachments */
+		if (1) {
+			AttachmentWorkStruct tmp;
+			tmp.i = 0;
+			tmp.a = a;
+			tmp.str = g_string_new (NULL);
+			g_mime_part_foreach (mp, add_inline_images_mpfunc, &tmp);
+			g_string_append (buf, tmp.str->str);
+			g_string_free (tmp.str, TRUE);
+		}
+			
 	}
 
 	/* cleanup */
@@ -500,20 +543,20 @@ text_get_article_headers (const Article * a, GString * buf)
 	g_string_append (buf, "<tr> <td width=\"20\" bgcolor=\"#008899\">&nbsp; </td><td><table>");
 
 	if (header_flags & UI_HEADER_SUBJECT) {
-		gchar * subject = make_safe_for_html (a->subject);
+		gchar * subject = pan_str_escape (article_get_subject(a));
 		g_string_sprintfa (buf, "<tr><td><b>%s</b></td></tr>", subject);
 		g_free (subject);
 	}
 	if (header_flags & UI_HEADER_AUTHOR) {
 		gchar * author = article_get_author_str (a);
-		replace_gstr (&author, make_safe_for_html(author));
+		replace_gstr (&author, pan_str_escape(author));
 		if (is_nonempty_string(a->author_addr))
 			replace_gstr (&author, g_strdup_printf("<a href=\"mailto:%s\">%s</a>", a->author_addr, author));
 		g_string_sprintfa (buf, "<tr><td><b>%s:</b> %s</td></tr>", _("From"), author);
 		g_free (author);
 	}
 	if (header_flags & UI_HEADER_MESSAGE_ID)
-		add_header (buf, _("Message-ID:"), a->message_id);
+		add_header (buf, _("Message-ID:"), article_get_message_id(a));
 	if (header_flags & UI_HEADER_REFERENCES)
 		add_header (buf, _("References:"), a->references);
 	if (header_flags & UI_HEADER_REPLY_TO)
@@ -537,14 +580,20 @@ text_get_article_headers (const Article * a, GString * buf)
 		Group * g = a->group;
 		gchar ** ids = g_strsplit (a->references, " ", -1);
 		int i;
+		int ref_nr = 0;
+
 		g_string_sprintfa (buf, "<b>%s</b>&nbsp;", _("References:"));
 		for (i=0; ids!=NULL && ids[i]!=NULL; ++i)
 		{
 			gchar * id = ids[i];
+
+			if (*id == 0) continue;
+
 			if (group_get_article_by_message_id (g, id) != NULL)
-				g_string_sprintfa (buf, "<a href=\"message-id:%s\">%d</a>&nbsp;", id, i+1);
+				g_string_sprintfa (buf, "<a href=\"message-id:%s\">%d</a>&nbsp;", id, 
+					++ref_nr);
 			else
-				g_string_sprintfa (buf, "%d&nbsp;", i+1);
+				g_string_sprintfa (buf, "%d&nbsp;", ++ref_nr);
 		}
 		g_strfreev (ids);
 	}
@@ -650,9 +699,7 @@ on_link_clicked (GtkHTML *html, const gchar *url, gpointer data)
 	}
 	else
 	{
-		pan_lock ();
-		gnome_url_show (url);
-		pan_unlock ();
+		pan_url_show (url);
 	}
 	debug_exit ("on_link_clicked");
 }
@@ -671,30 +718,6 @@ object_requested_cmd (GtkHTML *html, GtkHTMLEmbedded *eb, void *data)
 
 #else /* PLAIN TEXT */
 
-static void
-text_page_down_or_next_article (GtkWidget * w)
-{
-	GtkText * text = GTK_TEXT(w);
-        gfloat upper;
-	gfloat old_val;
-
-	debug_enter ("text_page_down_or_next_article");
-	g_return_if_fail (text != NULL);
-
-	/* calculate a page down */
-	old_val = text->vadj->value;
-	text->vadj->value += text->vadj->page_increment;
-	upper = text->vadj->upper - text->vadj->page_size;
-	text->vadj->value = CLAMP (text->vadj->value, 0.0, upper);
-
-	/* either go down a page or, if we're at the end, read the next article */
-	if (text->vadj->value>=0.0 && text->vadj->value!=old_val)
-		gtk_signal_emit_by_name (GTK_OBJECT (text->vadj), "value_changed");
-	else
-		articlelist_view_next ();
-
-	debug_exit ("text_page_down_or_next_article");
-}
 
 /**
 ***
@@ -822,7 +845,7 @@ text_set_raw_idle (gpointer data)
 	debug_enter ("text_set_raw_idle");
 
 	if (text == NULL)
-		text = g_strdup ("");
+		text = g_strdup (" ");
 
 	pan_lock_unconditional ();
 	if (1)
@@ -892,17 +915,6 @@ text_set_rot13 (gboolean on)
 
 
 
-void
-text_read_more (void)
-{
-	debug_enter ("text_read_more");
-
-	text_page_down_or_next_article (Pan.text);
-
-	debug_exit ("text_read_more");
-}
-
-
 /* Called when the widget is first drawn, puts our welcome text into view */
 static void
 realize_text_cb (GtkWidget *text, gpointer data)
@@ -938,22 +950,6 @@ text_button_press_cb (GtkWidget *widget, GdkEventButton *bevent)
 	debug_exit ("text_button_press_cb");
 }
 
-/* Process a keyboard input event in the text pane */
-static void
-text_key_press_cb (GtkWidget *widget, GdkEventKey *event)
-{
-#if 0
-	switch (event->keyval)
-	{
-		case GDK_KP_Space:
-		case GDK_space:
-			text_page_down_or_next_article (GTK_TEXT (Pan.text));
-			break;
-		default: 
-			break;
-	}
-#endif
-}
 
 /**
 ***  CURRENT ARTICLE
@@ -1054,7 +1050,7 @@ static int
 compare_pgchar_pparticle_msgid (const void * a, const void *b)
 {
 	const gchar * msgid_a = (const gchar*) a;
-	const gchar * msgid_b = (*(const Article**)b)->message_id;
+	const gchar * msgid_b = article_get_message_id (*(const Article**)b);
 	pan_warn_if_fail (is_nonempty_string(msgid_a));
 	pan_warn_if_fail (is_nonempty_string(msgid_b));
 	return pan_strcmp (msgid_a, msgid_b);
@@ -1079,7 +1075,7 @@ group_articles_removed_cb (gpointer call_obj,
 	{
 		gboolean exact_match = FALSE;
 
-		lower_bound (current_article->message_id,
+		lower_bound (article_get_message_id(current_article),
 		             removed->pdata,
 		             removed->len,
 			     sizeof(gpointer),
@@ -1112,11 +1108,14 @@ text_create (void)
 
 #ifdef USE_GTKHTML
 	Pan.text = gtk_html_new ();
+	/* this kludge kills gtkhtml's GDK_space handling -- we need it for space reading */
+	gtk_binding_entry_remove (gtk_binding_set_by_class(((GtkObject*)(Pan.text))->klass),
+	                          GDK_space, 0);
 	text_set_font (message_body_font);
 #else
         Pan.text = gtk_text_new (NULL, NULL);
         gtk_text_set_editable (GTK_TEXT(Pan.text), FALSE);
-        widget_set_font (GTK_WIDGET (Pan.text), message_body_font);
+        pan_widget_set_font (GTK_WIDGET (Pan.text), message_body_font);
         style = gtk_widget_get_style(GTK_WIDGET(Pan.text));
         style->text[0] = text_fg_color;
         style->base[0] = text_bg_color;                                                         
@@ -1134,8 +1133,6 @@ text_create (void)
 			    GTK_SIGNAL_FUNC(realize_text_cb), NULL);
 	gtk_signal_connect (GTK_OBJECT (Pan.text), "button_press_event",
 			    GTK_SIGNAL_FUNC(text_button_press_cb), NULL);
-	gtk_signal_connect (GTK_OBJECT (Pan.text), "key_press_event",
-			    GTK_SIGNAL_FUNC(text_key_press_cb), NULL);
 #ifdef USE_GTKHTML
         gtk_signal_connect (GTK_OBJECT (Pan.text), "url_requested",
                             GTK_SIGNAL_FUNC (url_requested), NULL);
@@ -1175,7 +1172,7 @@ text_create (void)
 	                  NULL);
 
 	/* watch for the articlelist group changing */
-	pan_callback_add (articlelist_group_changed,
+	pan_callback_add (articlelist_get_group_changed_callback(),
 	                  articlelist_group_changed_cb,
 			  NULL);
 
@@ -1239,6 +1236,7 @@ text_get_message_to_reply_to (void)
 	gboolean has_selection;
 	debug_enter ("text_get_message_to_reply_to");
 
+	pan_lock ();
 #ifdef USE_GTKHTML
 	{
 		GtkHTML * html = GTK_HTML(Pan.text);
@@ -1260,6 +1258,7 @@ text_get_message_to_reply_to (void)
 		}
 	}
 #endif
+	pan_unlock ();
  
 	/* no selection, so user is replying to whole message sans signature */
 	if (mm!=NULL && !has_selection)
@@ -1268,7 +1267,7 @@ text_get_message_to_reply_to (void)
 		gchar * pch = NULL;
 		replace_gstr (&body, g_mime_message_get_body (mm, FALSE, &is_html));
 		if (body != NULL)
-			pch = strstr (body, "\n-- \n");
+			pch = pan_strstr (body, "\n-- \n");
 		if (pch != NULL)
 			pch[1] = '\0';
 	}
@@ -1343,7 +1342,10 @@ text_set_show_all_headers (gboolean show)
 		_show_all_headers = show;
 		pan_callback_call (text_get_show_all_headers_changed_callback(), NULL, GINT_TO_POINTER(show));
 		text_refresh ();
+
+		pan_lock ();
 		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(text_menu_popup[5].widget), show);
+		pan_unlock ();
 	}
 
 	debug_exit ("text_set_show_all_headers");
@@ -1382,7 +1384,10 @@ text_set_mute_quoted (gboolean quoted)
 		_mute_quoted = quoted;
 		pan_callback_call (text_get_mute_quoted_changed_callback(), NULL, GINT_TO_POINTER(quoted));
 		text_refresh ();
+
+		pan_lock ();
 		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(text_menu_popup[6].widget), quoted);
+		pan_unlock ();
 	}
 
 	debug_exit ("text_set_mute_quoted");
@@ -1425,7 +1430,7 @@ void
 text_set_font (const gchar * font_name)
 {
 #ifndef USE_GTKHTML
-	widget_set_font (GTK_WIDGET(Pan.text), message_body_font);
+	pan_widget_set_font (GTK_WIDGET(Pan.text), message_body_font);
 #else
 	GtkHTML * html = GTK_HTML (Pan.text);
 
@@ -1448,3 +1453,93 @@ text_set_font (const gchar * font_name)
 #endif
 	text_refresh ();
 }
+
+/***
+****
+***/
+
+void
+update_body_pane_nolock (GtkWidget    * body_widget,
+                         const char   * body,
+		         gboolean       mute_quotes)
+{
+	GtkText * text;
+	gulong len;
+	gchar * buf = NULL;
+	debug_enter ("update_body_pane_nolock");
+
+	/* sanity checks */
+	g_return_if_fail (body_widget!=NULL);
+	g_return_if_fail (GTK_IS_TEXT(body_widget));
+
+	/* mute the quoted text, if desired */
+	if (mute_quotes)
+		body = buf = mute_quoted_text_in_body (body);
+
+	/* prepare the widget */
+	text = GTK_TEXT(body_widget);
+	gtk_text_freeze (text);
+	len = gtk_text_get_length (text);
+	gtk_text_set_point (text, len);
+
+	/* add the new body */
+	if (is_nonempty_string(body))
+	{
+       		GString * str = g_string_new (NULL);
+		GString * chunk = g_string_new (NULL);
+		gint chunk_quote = -1;
+
+		while (get_next_token_g_str (body, '\n', &body, str))
+		{
+			const gint is_quote = str->len && *str->str=='>' ? 1 : 0;
+			g_string_append_c (str, '\n');
+
+			if (chunk_quote==-1 || chunk_quote==is_quote)
+			{
+				g_string_append (chunk, str->str);
+			}
+			else if (chunk->len)
+			{
+				GdkColor * pFG = chunk_quote ? &text_quoted_color : &text_fg_color;
+				gtk_text_insert (text, NULL, pFG, NULL, chunk->str, chunk->len);
+				g_string_assign (chunk, str->str);
+			}
+
+			chunk_quote = is_quote;
+		}
+
+		if (chunk->len)
+		{
+			GdkColor * pFG = chunk_quote ? &text_quoted_color : &text_fg_color;
+			gtk_text_insert (text, NULL, pFG, NULL, chunk->str, chunk->len);
+		}
+
+		g_string_free (str, TRUE);
+		g_string_free (chunk, TRUE);
+	}
+
+	/* remove the old text, if any */
+	if (len != 0)
+		gtk_editable_delete_text (GTK_EDITABLE(body_widget), 0, len);
+
+	/* done with text widget */
+	gtk_text_thaw (text);
+
+	/* cleanup */
+	g_free (buf);
+	debug_exit ("update_body_pane_nolock");
+}
+
+void
+update_body_pane (GtkWidget    * body_widget,
+                  const char   * body,
+		  gboolean       mute_quotes)
+{
+	debug_enter ("update_body_pane");
+
+	pan_lock ();
+	update_body_pane_nolock (body_widget, body, mute_quotes);
+	pan_unlock ();
+
+	debug_exit ("update_body_pane");
+}
