@@ -23,8 +23,8 @@
 
 #include <gdk/gdk.h>
 #include <gtk/gtkwidget.h>
-#include "message-window.h"
-#include "pan-callback.h"
+#include <pan/base/pan-callback.h>
+#include <pan/message-window.h>
 
 /**
 ***  CURRENT ARTICLE
