@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team <pan@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,10 +26,12 @@
 
 #include <glib.h>
 
-#include "thread-watch.h"
-#include "gnksa.h"
-#include "prefs.h"
-#include "util.h"
+#include <pan/base/base-prefs.h>
+#include <pan/base/pan-glib-extensions.h>
+#include <pan/base/util-file.h>
+
+#include <pan/prefs.h>
+#include <pan/thread-watch.h>
 
 /*********************
 **********************  Defines / Enumerated types
@@ -175,6 +177,9 @@ thread_get_state (const Article * a)
 {
 	gint retval = THREAD_NOTHING;
 
+	/* sanity clause */
+	g_return_val_if_fail(a, retval);
+
 	/* make sure module's ready */
 	if (hash == NULL)
 		thread_init_module ();
@@ -198,8 +203,19 @@ thread_get_state (const Article * a)
 				g_free (id);
 			}
 		}
+
+		/* Maybe this Article has a broken References: header.
+		   Thanks to Adrian von Bidder for suggesting this.
+		   People posting broken References: headers should be flogged. */
+		if (retval==THREAD_NOTHING && a->parent!=NULL) {
+			Article * root = a->parent;
+			while (root->parent != NULL)
+				root = root->parent;
+			retval = thread_get_state (root);
+		}
 	}
 
+
 	return retval;
 }
 
@@ -239,6 +255,9 @@ thread_set_state (const Article * a, int state)
 		}
 		dirty = TRUE;
 	}
+
+	if (a->group != NULL)
+		a->group->articles_dirty = TRUE;
 }
 
 
@@ -294,7 +313,7 @@ thread_shutdown_module (void)
 		fprintf (fp, "# one record per line, comma-delimited fields\n");
 		fprintf (fp, "# each record has three fields:\n");
 		fprintf (fp, "#    state (0=nothing,1=watched,2=ignored)\n");
-		fprintf (fp, "#    start time (in secs since Jan 1, 1970)\n");
+		fprintf (fp, "#    when this state was set, in secs since Jan 1, 1970.  These states expire after 1 month.\n");
 		fprintf (fp, "#    message-id of parent thread\n");
 		g_hash_table_foreach (hash, thread_write_line, fp);
 		fclose (fp);
